package lbl.akenti.certs;

import java.text.ParseException;
import java.security.PublicKey;
import java.security.GeneralSecurityException;
import java.io.IOException;
import org.w3c.dom.Element;
import lbl.akenti.certs.elements.AkentiIssuer;
import lbl.akenti.certs.elements.AkentiInputStream;
import lbl.akenti.certs.elements.AkentiOutputStream;
import lbl.akenti.util.UTCTime;

/**
  * This abstract class is the superclass of all certificates.
  *
  * @see AkentiOutputStream
  * @see AkentiInputStream
  * @see AkentiIssuer
  * @see UTCTime
  *
  * @author Abdelilah Essiari
  **/

public abstract class Certificate {

  protected String type;

  /**
    * Constructs this <i>AkentiCertificate</i>.
    *
    * @param type certificate type
    */
  protected Certificate(String type) {
    this.type = type;
  }

  /**
    * Returns this <i>AkentiCertificate</i>'s type.
    */
  public String getType() {
    return type;
  }

  /**
    * Returns the approriate hash string whose digest will be used to
    * locate certificates.
    *
    **/
  public abstract String getHashString();

  /**
    * Returns the version number.
    */
  public abstract int getVersion();

  /**
    * Returns this <i>AkentiCertificate</i>'s unique id.
    */
  public abstract String getUniqueID();

  /**
    * Returns this <i>AkentiCertificate</i>'s issuer.
    */
  public abstract AkentiIssuer getIssuer();

  /**
    * Returns the start validity date.
    */
  public abstract UTCTime getNotBefore();

  /**
    * Returns the end validity date.
    */
  public abstract UTCTime getNotAfter();

  /**
    */
  public boolean checkValidity(UTCTime utc) {
    return getNotBefore().compareTo(utc) <= 0 
           && utc.compareTo(getNotAfter()) <= 0;
  }

  /**
    * Returns the name of the signature algorithm.
    */
  public abstract String getSignatureAlg();

  /**
    * Returns the name of the canonalization algorithm.
    */
  public abstract String getCanonAlg();

  /**
    * Returns the signature.
    */
  public abstract byte[] getSignature();

  /**
    * Returns the signable part of this certificate.
    *
    */
  public abstract byte[] getSignable() throws IOException;

  /**
    * Verifies the signature.
    */
  public abstract boolean verify(PublicKey publicKey)
    throws IOException, GeneralSecurityException;
  
  /**
    * Returns an instance of this certificate.
    */
  protected abstract Certificate newInstance(); 

  public abstract void writeObject(AkentiOutputStream drain) 
    throws IOException;

  protected abstract void readObject(AkentiInputStream source)
    throws IOException, ParseException;

  protected  abstract void fromXML(Element cert) throws ParseException;

  public abstract void toXML(Element cert);

}
