package lbl.akenti.certs;

import java.io.IOException;
import java.text.ParseException;
import java.util.ArrayList;
import lbl.akenti.certs.elements.AkentiInputStream;
import lbl.akenti.certs.elements.AkentiOutputStream;
import lbl.akenti.certs.elements.AkentiPrincipal;
import lbl.akenti.certs.elements.AttributeInfo;
import lbl.akenti.certs.elements.Condition;
import lbl.akenti.certs.elements.Directory;
import lbl.akenti.certs.elements.DistinguishedName;
import lbl.akenti.util.ParsingException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;


public class UseCondCertificate extends AkentiCertificate {
  private String resource;
  private boolean localScope;
  private boolean critical;
  private Condition condition;
  private String[] actions;
  
  public UseCondCertificate() {
    super("UseCondition");
    resource = "NO_RESOURCE";
    localScope = true;
    critical = true;
    condition = new Condition();
    actions = new String[0];
  }
  
  public UseCondCertificate(AkentiInputStream in) throws IOException, ParseException {
    super(in);
  }
  
  public String getHashString() {
    return getResource();
  }
  
  protected void setResource(String resource) {
    if (resource.length() == 0) {
      throw new IllegalArgumentException();
    }
    
    this.resource = resource;
  }
  
  public String getResource() {
    return resource;
  }
  
  protected void setLocalScope(boolean localScope) {
    this.localScope = localScope;
  }
  
  public boolean isScopeLocal() {
    return localScope;
  }
  
  protected void setCritical(boolean critical) {
    this.critical = critical;
  }
  
  public boolean isCritical() {
    return critical;
  }
  
  protected void setCondition(Condition condition) {
    if (condition == null) {
      throw new IllegalArgumentException();
    }
    
    this.condition = condition;
  }
  
  public Condition getCondition() {
    return condition;
  }
  
  public String getConstraint() {
    return condition.getConstraint();
  }
  
  public AttributeInfo[] getAttributeInfos() {
    return condition.getAttributeInfos();
  }
  
  protected void setActions(String[] actions) {
    for (int i=0; i < actions.length; i++) {
      if (actions[i].length() == 0) {
        throw new IllegalArgumentException();
      }
    }
    
    this.actions = actions;
  }
  
  public String[] getActions() {
    return actions;
  }
  
  protected Certificate newInstance() {
    return new UseCondCertificate();
  }
  
  protected void
  doWrite(AkentiOutputStream drain) throws IOException {
    drain.write(resource);
    drain.write(' ');
    drain.write(localScope);
    drain.write(' ');
    drain.write(critical);
    drain.write(' ');
    condition.writeObject(drain);
    drain.write(' ');
    drain.write(actions.length);
    
    for (int i=0; i < actions.length;  i++) {
      drain.write(' ');
      drain.write(actions[i]);
    }
  }
  
  protected void
  doRead(AkentiInputStream source) throws IOException, ParseException {
    resource = source.readString();
    localScope = source.readBoolean();
    critical = source.readBoolean();
    condition = new Condition(source);
    
    int size = source.readInt();
    
    if (size < 0) {
      throw new ParsingException("actions.size < 0");
    }
    
    actions = new String[size];
    
    for (int i=0; i < size; i++) {
      actions[i] = source.readString();
    }
  }
  
  protected void doToXML(Element ucCertInfo) {
    ucCertInfo.setAttribute("scope", localScope? "local" : "sub-tree");
    ucCertInfo.setAttribute("critical", critical? "true" : "false");
    
    Document doc = ucCertInfo.getOwnerDocument();
    Element temp = doc.createElement("ResourceName");
    
    temp.appendChild(doc.createTextNode(resource));
    ucCertInfo.appendChild(temp);
    
    temp = doc.createElement("Condition");
    condition.toXML(temp);
    ucCertInfo.appendChild(temp);
    
    for (int i = 0; i < actions.length; ++i) {
      temp = doc.createElement("Action");
      temp.appendChild(doc.createTextNode(actions[i]));
      ucCertInfo.appendChild(temp);
    }
  }
  
  protected void
  doFromXML(Element ucCertInfo) throws ParseException {
    String temp = ucCertInfo.getAttribute("scope");
    
    if (temp == null) {
      throw new ParsingException();
    } else if (temp.equals("local")) {
      localScope = true;
    } else if (temp.equals("sub-tree")) {
      localScope = false;
    } else {
      throw new ParsingException();
    }
    
    temp = ucCertInfo.getAttribute("critical");
    
    if (temp == null) {
      throw new ParsingException();
    } else if (temp.equals("true")) {
      critical = true;
    } else if (temp.equals("false")) {
      critical = false;
    } else {
      throw new ParsingException();
    }
    
    Node child = ucCertInfo.getFirstChild();
    
    if (child == null || child.getNodeType() != Node.ELEMENT_NODE) {
      throw new ParsingException();
    }
    
    Node node = child.getFirstChild();
    
    if (node == null) {
      throw new ParsingException();
    }
    
    resource = node.getNodeValue();
    
    if (resource == null) {
      new ParsingException();
    }
    
    child = child.getNextSibling();
    
    if (child == null || child.getNodeType() != Node.ELEMENT_NODE) {
      throw new ParsingException();
    }
    
    condition = new Condition((Element) child);
    
    child = child.getNextSibling();
    
    if (child == null) {
      actions = new String[0];
      return;
    }

    ArrayList list = new ArrayList();

    do {
      if (child.getNodeType() != Node.ELEMENT_NODE) {
        throw new ParsingException();
      }

      node = child.getFirstChild();
    
      if (node == null) {
        throw new ParsingException();
      }
    
      list.add(node.getNodeValue());
      child = child.getNextSibling();
    } while (child != null);
    
    actions = (String[]) list.toArray(new String[list.size()]);
  }
      
  protected void doToText(StringBuffer sbuf) {
    String scope = (localScope) ? "'local'" : "'subtree'";
    
    String isCritical = (critical) ? "'true'" : "'false'";
    
    sbuf.append("Resource: " + resource + "\n");
    sbuf.append("Use Condition: scope=" + scope + " critical=" + isCritical + "\n");
    sbuf.append("  Constraint: " + getConstraint() + "\n");
    sbuf.append("  Action: ");
    
    String[] actions = getActions();
    sbuf.append(actions[0]);
    for (int i=1; i < actions.length; i++) {
      sbuf.append(", " + actions[i]);
    }
    sbuf.append("\n");
    
    sbuf.append("  Attribute Infos:\n");
    
    AttributeInfo[] ais = this.getAttributeInfos();
    for (int i=0; i < ais.length; i++) {
      
      sbuf.append("    " + ais[i].getAttribute() + " = " + ais[i].getValue());
      sbuf.append(", type = " + AttributeInfo.TYPES[ais[i].getType()] + "\n");
      
      if (ais[i].getType() == AttributeInfo.X509) {
        
        DistinguishedName[] cas = ais[i].getCAs();
        for (int j=0; j < cas.length; j++) {
          sbuf.append("      CA: " + cas[j] + "\n");
        }
        
      } else if (ais[i].getType() == AttributeInfo.AKENTI ||
      ais[i].getType() == AttributeInfo.EXT_AUTH) {
        
        AkentiPrincipal[] ap = null;
        if (ais[i].getType() == AttributeInfo.AKENTI ) {
          ap = ais[i].getIssuers();
        } else {
          ap = ais[i].getAuthorities();
        }
        
        for (int j=0; j < ap.length; j++) {
          sbuf.append("      Name: " + ap[j].getName() + "\n");
          sbuf.append("      CA: " + ap[j].getGuarantor() + "\n");
        }
        
        Directory[] dirs = ais[i].getDirectories();
        if (dirs.length != 0) {
          sbuf.append("      Directories:\n");
          
          for (int k=0; k < dirs.length; k++) {
            sbuf.append("        " + dirs[k].toString() + "\n");
          }
        }
        
        if (ais[i].getType() == AttributeInfo.EXT_AUTH) {
          String[] args = ais[i].getArguments();
          
          if (args.length != 0) {
            sbuf.append("      Arguments:\n");
            
            for (int k=0; k < args.length; k++) {
              sbuf.append("        " + args[k] + "\n");
            }
          }
        }
        
      }
    }
  }
}