package lbl.akenti.certs.elements;

import java.io.PrintStream;
import java.io.OutputStream;
import java.io.IOException;
import java.util.StringTokenizer;

/**
  * An <i>AkentiOutputStream</i> object is used to serialize all
  * types of objects using a simple scheme to write the objects'
  * string reprsentations for primitives types such as int(s) 
  * and char(s).
  *
  * For string objects containing blank spaces those spaces are escaped 
  * by preceeding them by a back slash. Escaping the blank space allows
  * the deserializing process to distinguish between blank spaces that
  * are part of an object being deserialized and the blank spaces used
  * as delimiters between elements. Only the object is written to the 
  * ostream. The delimiting blank space between the ojbects must be written
  * by the user of this class.
  *
  * Note:
  *
  *   Blank spaces at the beginning and at the end of an object's
  *   representation are ignored. Those in the middle are escaped
  *   and if there is a sequence of blanks in the middle,
  *   only one is escaped and the others are ignored.
  *
  * Example1:
  *
  * "/C=US/CN=John Doe" will be written as "/C=US/CN=John\ Doe"
  *
  * Example2:
  *
  *   Deserializing "Hello\ John Bye\ John" will result in
  *   two strings "Hello John" and "Bye John"
  *
  * It is recommended to use an <i>AkentiInputStream</i> object
  * to deserialize objects.
  *
  * @see AkentiInputStream
  *
  * @version 1.1 00/05/01
  * @author Srilekha Mudumbai
  * @author Abdelilah Essiari
  **/

final public class AkentiOutputStream {

  private  PrintStream os;

  public AkentiOutputStream(OutputStream os) {
    this.os = new PrintStream(os);
  }

  public void write(byte[] buf) throws IOException {
    write(buf, 0, buf.length);
  }

  public void write(byte[] buf, int off, int len) throws IOException {
    os.write(buf, off, len);

    if (os.checkError()) {
      throw new IOException("AkentiOutputStream:write(byte[], int, int) failed");
    } 
  }

  public void write(boolean b) throws IOException {
    int temp = (b == true? 1 : 0);

    os.print(temp);

    if (os.checkError()) {
      throw new IOException("AkentiOutputStream:write(boolean) failed");
    } 
  }

  public void write(int b) throws IOException {
    os.print(b);

    if (os.checkError()) {
      throw new IOException("AkentiOutputStream:write(int) failed");
    } 
  }

  public void write(char c) throws IOException {
    os.print(c);

    if (os.checkError()) {
      throw new IOException("AkentiOutputStream:write(char) failed");
    } 
  }

  public void write(char[] c) throws IOException {
    os.print(c);

    if (os.checkError()) {
      throw new IOException("AkentiOutputStream:write(char[]) failed");
    } 
  }

  public void write(byte b) throws IOException {
    os.write(b);

    if (os.checkError()) {
      throw new IOException("AkentiOutputStream:write(byte) failed");
    } 
  }

  // replace ' ' by '\\ ' for all the incoming strings
  public void write(String s) throws IOException {
    StringTokenizer tokenizer = new StringTokenizer(s, " ");

    os.print(tokenizer.nextToken());
    
    while (tokenizer.hasMoreTokens()) {
      os.print("\\ ");
      os.print(tokenizer.nextToken());
    } 

    if (os.checkError()) {
      throw new IOException("AkentiOutputStream:write(String) failed");
    } 
  }

  public void write(double d) throws IOException {
    os.print(d);

    if (os.checkError()) {
      throw new IOException("AkentiOutputStream:write(double) failed");
    } 
  }

  public void write(float f) throws IOException {
    os.print(f);

    if (os.checkError()) {
      throw new IOException("AkentiOutputStream:write(float) failed");
    } 
  }

  public void write(long l) throws IOException {
    os.print(l);

    if (os.checkError()) {
      throw new IOException("AkentiOutputStream:write(long) failed");
    } 
  }

  public void write(short s) throws IOException {
    os.print(s);

    if (os.checkError()) {
      throw new IOException("AkentiOutputStream:write(short) failed");
    } 
  }

  public void flush() throws IOException {
    os.flush();

    if (os.checkError()) {
      throw new IOException("AkentiOutputStream:flush() failed");
    } 
  }

  public void close() throws IOException {
    os.close();

    if (os.checkError()) {
      throw new IOException("AkentiOutputStream:close() failed");
    } 
  }
}
