package lbl.akenti.certs.elements;

import java.io.IOException;
import java.text.ParseException;

import lbl.akenti.util.ParsingException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.w3c.dom.Node;

/**
  * An <i>AttributeInfo</i> consists mainly of an attribute, a value,
  * and possibly a list of cas/issuers/authorities depending on the
  * attribute type.
  *
  * <BR>
  *
  * Note that these attribute/value pairs can be resource-specific
  * or user-specific.
  *
  * <pre>
  *
  * We have considered four types of attributes,
  *
  *  1) an X509 attribute.
  *
  *     This attribute/value pair puts a constraint on the
  *     dn of a user accessing a resource.
  *
  *     Example: OU=LBL
  *
  *     We ask for a non-empty list of the cas that we
  *     are willing to accept.
  *
  *     For now the cas have to be the immediate cas of
  *     the user accessing the resource.
  *
  *  2) an AKENTI attribute.
  *
  *     This is an Akenti Certificate based attribute.
  *     An attribute certificate is fetched and used
  *     to evaluate this attribute/value pair.
  *
  *     Example:  group=SOME_GROUP
  *
  *     We ask for a non-empty list of issuers
  *     that can issue such attribute certificates.
  *
  *  3) an EXT_AUTH attribute.
  *
  *     This attribute/value pair is evaluated or attested
  *     by some authority. Akenti Engine may be configured
  *     to evaluate such attribute/value pairs.
  *
  *     Example: time=10:00  and SomeTimeAuthority
  *
  *     We ask for a non-empty list of authorities that help
  *     evaluate such attribute/value pairs.
  *
  *  4) a SYSTEM attribute.
  *
  *     This is the simplest of all attributes.
  *     Akenti Engine may be configured to evaluate such
  *     such attribute/value pairs.
  *
  *     Example: DiscSpace=10 MB
  *
  * AttributeInfo {
  *   int                                type; // { SYSTEM, X509, AKENTI , EXT_AUTH }
  *   string                        attribute; // name of attribute
  *   string                            value; // value of attribute
  *   vector&lt;DistinguishedName&gt;     cas; // X509
  *   vector&lt;AkentiPrincipal&gt;       aps; // AKENTI/EXT_AUTH attribute certs issuers
  *   vector&lt;Directory&gt;            dirs; // AKENTI used to look up attribute certs
  *                                            // EXT_AUTH used to contact authoriies
  *   vector&ltstring&gt                 args; // EXT_AUTH args passed to the authorities
  * }
  *
  * The writeObject output is:
  *
  * For a SYSTEM attribute:
  *     0 attribute value
  *
  * For an X509 attribute:
  *     1 attribute value caCount ca+
  *
  * For an AKENTI attribute:
  *     2 attribute value apCount ap+ dirCount dir*
  *
  * For an EXT_AUTH attribute:
  *     3 attribute value apCount ap+ dirCount dir* argCount arg*
  *
  * Where + means one or more and * means 0 or more
  *
  * </pre>
  *
  * @see AkentiObject
  * @see AkentiPrincipal
  * @see Directory
  * @see DistinguishedName
  *
  * @version 1.1 02/05/01
  *
  * @author Srilekha Mudumbai
  * @author Abdelilah Essiari
  */
public final class AttributeInfo extends AkentiObject {

  public static final int SYSTEM = 0;
  public static final int X509 = 1;
  public static final int AKENTI = 2;
  public static final int EXT_AUTH = 3;

  public static final String[] TYPES = { 
    "SYSTEM", "X509", "AKENTI", "EXT_AUTH"
  };

  private int type;
  private String attribute;
  private String value;
  private DistinguishedName[] cas;
  private AkentiPrincipal[] aps;
  private Directory []dirs;
  private String[] args;

  /**
    * Constucts this <i>AttributeInfo</i> using default values.
    */
  public AttributeInfo() {
    this("NO_ATTRIBUTE", "NO_VALUE");
  }

  /**
    * Constructs a <i>SYSTEM AttributeInfo</i>. 
    **/
  public AttributeInfo(String attr, String val) {
    setType(SYSTEM);
    setAttribute(attr);
    setValue(val);
  }

  /**
    * Constructs a <i>X509 AttributeInfo</i>. 
    **/
  public AttributeInfo(String attr, String val, DistinguishedName[] cas) {
    setType(X509);
    setAttribute(attr);
    setValue(val);
    
    if (cas.length == 0) {
      throw new IllegalArgumentException();
    }

    this.cas = cas;
  }

  /**
    * Constructs a <i>Akenti AttributeInfo</i>. 
    **/
  public AttributeInfo(String attr, String val, AkentiPrincipal[] aps, Directory []dirs) {
    setType(AKENTI);
    setAttribute(attr);
    setValue(val);

    if (aps.length == 0) {
      throw new IllegalArgumentException();
    }

    this.aps = aps;
    this.dirs = dirs == null? new Directory[0] : dirs;
  }

  /**
    * Constructs a <i>EXT_AUTH AttributeInfo</i>. 
    **/
  public 
  AttributeInfo(String attr, String val, AkentiPrincipal[] aps, Directory []dirs, String[] args) {
    setType(EXT_AUTH);
    setAttribute(attr);
    setValue(val);

    if (aps.length == 0) {
      throw new IllegalArgumentException();
    }

    this.aps = aps;
    this.dirs = dirs == null? new Directory[0] : dirs;
    this.args = args == null? new String[0] : args;
  }

  /**
    * Initializes this <i>AttributeInfo<i> from a <i>AkentiInputStream</i>.
    */
  public
  AttributeInfo(AkentiInputStream source) throws IOException, ParseException {
    readObject(source);
  }

  /**
    * Initializes this <i>AttributeInfo<i> from a XML <i>Element</i>.
    */
  public
  AttributeInfo(Element element) throws ParseException {
    fromXML(element);
  }

  /**
    * Sets the <i>type</i>. 
    */
  private void setType(int type) {
    if (type < SYSTEM || type > EXT_AUTH) {
      throw new IllegalArgumentException();
    }
    
    this.type = type;
  }

  /**
    * Returns <i>true</i> if this <i>AttributeInfo<i> has the type
    * specified by the argument <i>type</i>.
    */
  public boolean hasType(int type) {
    return this.type == type;
  }

  /**
    * Returns the <i>type</i>.
    */
  public int getType() {
    return type;
  }

  /**
    * Sets the <i>attribute</i>. 
    */
  private void setAttribute(String attribute) {
    if (attribute.length() == 0) {
      throw new IllegalArgumentException();
    }

    this.attribute = attribute;
  }

  /**
    * Returns <i>true</i> if this <i>AttributeInfo<i> has the 
    * attribute specified by the argument <i>attribute</i>.
    */
  public boolean hasAttribute(String attribute) {
    return this.attribute.equals(attribute);
  }

  /**
    * Returns the <i>attribute</i>.
    */
  public String getAttribute() {
    return attribute;
  }

  /**
    * Sets the <i>value</i>. 
    */
  private void setValue(String value) {
    if (value.length() == 0) {
      throw new IllegalArgumentException();
    }

    this.value = value;
  }

  /**
    * Returns <i>true</i> if this <i>AttributeInfo<i> has the 
    * value specified by the argument <i>value</i>.
    */
  public boolean hasValue(String value) {
    return this.value.equals(value);
  }

  /**
    * Returns the <i>value</i>.
    */
  public String getValue() {
    return value;
  }

  /**
    * Returns <i>true<i> if the <i>ca</i> is listed 
    * as one of the cas for this <i>AttributeInfo</i>.
    */
  public boolean hasCA(DistinguishedName ca) {
    if (type != X509) {
      throw new Error("AttributeInfo:hasCA:not x509 attribute");
    }

    boolean ret = ca.equals(cas[0]);
 
    for (int i = 1; !ret && i < cas.length; ++i) {
      ret = ca.equals(cas[i]);
    }

    return ret;
  }

  /**
    * Returns the <i>X509</i> cas.
    */
  public DistinguishedName[] getCAs() {
    if (type != X509) {
      throw new Error ("wrong type");
    }

    return cas;
  }

  /**
    * Returns <i>true<i> if the <i>issuer</i> is listed 
    * as one of the issuers for this <i>AttributeInfo</i>.
    */
  public boolean hasIssuer(AkentiPrincipal issuer) {
    if (type != AKENTI) {
      throw new Error("wrong type");
    }

    boolean ret = issuer.equals(aps[0]);

    for (int i = 1; !ret && i < aps.length; ++i) {
      ret = issuer.equals(aps[i]);
    }

    return ret;
  }

  /**
    * Returns the <i>AKENTI</i> issuers.
    */
  public AkentiPrincipal[] getIssuers() {
    if (type != AKENTI) {
      throw new Error("wrong type");
    }

    return aps;
  }

  /**
    * Returns <i>true<i> if the <i>authority</i> is listed 
    * as one of the authorities for this <i>AttributeInfo</i>.
    */
  public boolean hasAuthority(AkentiPrincipal authority) {
    if (type != EXT_AUTH) {
      throw new Error("wrong type");
    }

    boolean ret = authority.equals(aps[0]);

    for (int i = 1; !ret && i < aps.length; ++i) {
      ret = authority.equals(aps[i]);
    }

    return ret;
  }

  /**
    * Returns the <i>EXT_AUTH</i> authorities.
    */
  public AkentiPrincipal[] getAuthorities() {
    if (type != EXT_AUTH) {
      throw new Error ("wrong type");
    }

    return aps;
  }

  /**
    * Returns the <i>attribute</i> directories.
    */
  public Directory[] getDirectories() {
    if (type != EXT_AUTH && type != AKENTI) {
      throw new Error ("wrong type");
    }

    return dirs;
  }

  /**
    * Returns the arguments.
    */
  public String[] getArguments() {
    if (type != EXT_AUTH) {
      throw new Error ("wrong type");
    }

    return args;
  }

  /**
    * Tests for equality. Useful in containers such as vectors or sets.
    */
  public boolean equals(Object obj) {
    if (obj instanceof AttributeInfo) {
      AttributeInfo ainfo = (AttributeInfo) obj;

      return 
        ainfo.hasType(type) && ainfo.hasAttribute(attribute) && ainfo.hasValue(value);
    }

    return false;
  }
	
  /**
    * Initializes this <i>AttributeInfo</i> from a <i>AkentiInputStream</i>.
    */
  private void 
  readObject(AkentiInputStream source) throws IOException, ParseException {
    type = source.readInt();
    attribute = source.readString();
    value = source.readString();

    if (type == SYSTEM) {
      return;
    } 

    int temp = source.readInt();

    if (temp <= 0) {
      throw new ParsingException("cas/issuers/auths size <= 0");
    }

    if (type == X509) {
      cas = new DistinguishedName[temp];

      for (int i = 0; i < temp; i++) {
        cas[i] = new DistinguishedName(source);
      }

      return;
    } else {
      aps = new AkentiPrincipal[temp];

      for (int i = 0; i < temp; i++) {
        aps[i] = new AkentiPrincipal(source);
      }
    }

    temp = source.readInt();

    if (temp < 0) {
      throw new ParsingException("dirs.size < 0");
    }

    dirs = new Directory[temp];

    for (int i = 0; i < temp; i++) {
      dirs[i] = new Directory(source);
    }

    if (type == AKENTI) {
      return;
    }

    temp = source.readInt();

    if (temp < 0) {
      throw new ParsingException("args.size < 0");
    }

    args = new String[temp];

    for (int i = 0; i < temp; i++) {
      args[i] = source.readString();
    }
  }

  /**
    * Writes this <i>AttributeInfo</i> to a <i>AkentiOutputStream</i>.
    */
  public void 
  writeObject(AkentiOutputStream drain) throws IOException {
    drain.write(type);
    drain.write(' ');
    drain.write(attribute);
    drain.write(' ');
    drain.write(value);

    if (type == SYSTEM) {
      return;
    } 
   
    drain.write(' ');

    if (type == X509) {
      drain.write(cas.length);

      for (int i=0; i < cas.length; ++i) {
        drain.write(' ');
        cas[i].writeObject(drain);
      }
      
      return;
    }

    drain.write(aps.length);

    for (int i=0; i < aps.length; ++i) {
      drain.write(' ');
      aps[i].writeObject(drain);
    }

    drain.write(' ');
    drain.write(dirs.length);
    
    for (int i= 0; i < dirs.length; i++) {
      drain.write(' ');
      dirs[i].writeObject(drain);
    }

    if (type == AKENTI) {
      return;
    }

    drain.write(' ');
    drain.write(args.length);

    for (int i = 0; i < args.length; ++i) {
      drain.write(' ');
      drain.write(args[i]);
    }
  }

  /**
    * Populates the XML element <i>info</i>.
    */
  public void toXML(Element info) {
    info.setAttribute("type", TYPES[type]);

    Document doc = info.getOwnerDocument();
    Element temp = doc.createElement("AttrName");

    temp.appendChild(doc.createTextNode(attribute));
    info.appendChild(temp);
    temp =  doc.createElement("AttrValue");
    temp.appendChild(doc.createTextNode(value));
    info.appendChild(temp);

    if (type == SYSTEM) {
      return;
    }

    if (type == X509) {
      for (int i=0; i < cas.length; i++) {
        temp = doc.createElement("CADN");
        cas[i].toXML(temp);
        info.appendChild(temp);
      }

      return;
    } 

    for (int i = 0; i < aps.length; i++) {
      temp = doc.createElement("Principal");
      aps[i].toXML(temp);
      info.appendChild(temp);
    }

    if (dirs.length > 0) {
      Element attrDirs = doc.createElement("AttrDirs");
    
      for (int i=0; i < dirs.length; i++) {
        temp = doc.createElement("URL");
        dirs[i].toXML(temp);
        attrDirs.appendChild(temp);
      }

      info.appendChild(attrDirs);
    }

    if (type == AKENTI) {
      return;
    }

    for (int i=0; i < args.length; i++) {
      Element arg = doc.createElement("ExtArg");
 
      arg.appendChild(doc.createTextNode(args[i]));
      info.appendChild(arg);
    }
  }

  /**
    * Initializes this <i>AttributeInfo</i> from the XML element <i>ap</i>.
    */
  private void fromXML(Element info) throws ParseException {
    String rep = info.getAttribute("type");
    int idx;
  
    for (idx = 0; idx < TYPES.length; idx++) {
      if (rep.equals(TYPES[idx])) {
        break;
      }
    } 

    if (idx == TYPES.length) {
      throw new ParsingException("unknow type:" + type);
    }
    
    type = idx;

    Node child = info.getFirstChild();
    
    if (child == null) {
      throw new ParsingException();
    }

    Node temp = child.getFirstChild();

    if (temp == null) {
      throw new ParsingException();
    }

    if ((attribute = temp.getNodeValue()) == null) {
      throw new ParsingException();
    } else if ((child = child.getNextSibling()) == null) {
      throw new ParsingException();
    } else if ((temp = child.getFirstChild()) == null) {
      throw new ParsingException();
    } else if ((value = temp.getNodeValue()) == null) {
      throw new ParsingException();
    } else if (type == SYSTEM) {
      return;
    }

    NodeList list;
    int len;

    if (type == X509) {
      list = info.getElementsByTagName("CADN");

      if ((len = list.getLength()) == 0) {
        throw new ParsingException("no cas found");
      }
   
      cas = new DistinguishedName[len];
   
      for (int i= 0;  i < len; ++i) {
        cas[i] = new DistinguishedName((Element) (child = list.item(i)));
      }

      return;
    }

    list = info.getElementsByTagName("Principal");

    if ((len = list.getLength()) == 0) {
      throw new ParsingException("no principals found");
    }

    aps = new AkentiPrincipal[len];

    for (int i = 0; i < len; ++i) {
      aps[i] = new AkentiPrincipal((Element) (child = list.item(i)));
    }

    if ((child = child.getNextSibling()) == null) {
      dirs = new Directory[0];
      
      if (type == EXT_AUTH) {
        args = new String[0];
      }

      return;
    } 

    if (child.getNodeName().equals("AttrDirs")) {
      if (child.getNodeType() != Node.ELEMENT_NODE) {
        throw new ParsingException();
      }

      list = ((Element) child).getElementsByTagName("URL");

      if ((len = list.getLength()) == 0) {
        throw new ParsingException();
      }

      dirs = new Directory[len];

      for (int i = 0; i < len; ++i) {
        dirs[i] = new Directory((Element) list.item(i));
      } 
    } else {
      dirs = new Directory[0];
    }
      
    if (type == AKENTI) {
      return;
    }

    list = info.getElementsByTagName("ExtArg");
    args = new String[len = list.getLength()];

    for (int i= 0; i < len; ++i) {
      if ((temp = list.item(i).getFirstChild()) == null) {
        throw new ParsingException();
      } else if ((args[i] = temp.getNodeValue()) == null) {
        throw new ParsingException();
      }
    }
  }

  /**
    * Returns debugging info.
    */
  public String paramString() {
    StringBuffer sbuf = new StringBuffer();

    sbuf.append("[");
    sbuf.append("Type");
    sbuf.append("=");
    sbuf.append(TYPES[type]);
    sbuf.append(",");
    sbuf.append("ATTRIBUTE");
    sbuf.append("=");
    sbuf.append(attribute);
    sbuf.append(",");
    sbuf.append("VALUE");
    sbuf.append("=");
    sbuf.append(value);
    sbuf.append("]");
    return sbuf.toString();
  }
}
