package lbl.akenti.certs.elements;

import java.io.IOException;
import java.text.ParseException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import lbl.akenti.util.ParsingException;
import lbl.akenti.parsers.Expression;

/**
  * Holds a boolean expression and a vector of <i>AttributeInfo</i>s
  * that describe the attribute/value pairs contained in the expression.
  *
  * <pre>
  * Condition {
  *    string constraint;
  *    vector&lt;AttributeInfo&gt; attributeInfos;
  * }
  * </pre>
  *
  * @see AttributeInfo
  * @see Expression
  *
  * @version 1.1 03/05/01
  *
  * @author Srilekha Mudumbai
  * @author Abdelilah Essiari
  **/

public final class Condition extends AkentiObject {
  private String constraint;
  private AttributeInfo[] attrInfos;

  /**
    * Constructs using default values.
    */
  public Condition() {
    this(null, null);
  }

  /**
    */
  public Condition(String constraint) {
    this(constraint, null);
  }

  /**
    * Constructs this <i>Condition</i>.
    */
  public Condition(String constraint, AttributeInfo[] attrInfos) {
    if (constraint == null || constraint.length() == 0) {
      constraint = "()";
    }

    if (attrInfos == null) {
      attrInfos = new AttributeInfo[0];
    }

    this.constraint = constraint;
    this.attrInfos = attrInfos;
  }

  /**
    * Constructs this <i>Condition</i>.
    *
    * @param source stream used to initialize this object.
    */
  public
  Condition(AkentiInputStream source) throws IOException, ParseException {
    readObject(source);
  }

  /**
    * Constructs this <i>Condition</i>.
    *
    * @param element XML element used to parse this object.
    */
  public
  Condition(Element element) throws ParseException {
    fromXML(element);
  }

  /**
    * Returns the constraint.
    */
  public String getConstraint() {
    return constraint;
  }

  /**
    */
  public Expression parseConstraint() {
    try {
      // XXXXX
      return new Expression(getConstraint()); 
    } catch(ParseException ex) {
      throw new RuntimeException();
    }
  } 

  /**
    * Returns the <i>AttributeInfo</i>s.
    */
  public AttributeInfo[] getAttributeInfos() {
    return attrInfos;
  }

  /**
    * Writes this <i>Condition</i> to a <i>AkentiOutputStream</i>.
    */
  public void writeObject(AkentiOutputStream drain) throws IOException {
    drain.write(constraint);

    drain.write(' ');
    drain.write(attrInfos.length);

    for (int i = 0; i < attrInfos.length; i++) {
      drain.write(' ');

      attrInfos[i].writeObject(drain);
    }
  }

  /**
    * Initializes this <i>Condition</i> from a <i>AkentiInputStream</i>.
    */
  private void 
  readObject(AkentiInputStream source) throws IOException, ParseException {
    constraint = source.readString();
    
    // TODO: FINISH THIS
    new lbl.akenti.parsers.Expression(constraint);

    int size = source.readInt();

    if (size < 0) {
      throw new ParsingException("attrinfos.size = " + size);
    }

    attrInfos = new AttributeInfo[size];
    
    for (int i = 0; i < size; i++) {
      attrInfos[i] = new AttributeInfo(source);
    }
  }

  /**
    * Populates the XML element <i>cond</i>.
    */
  public void toXML(Element cond) {
    Document doc = cond.getOwnerDocument();
    Element temp = doc.createElement("Constraint");

    temp.appendChild(doc.createTextNode(constraint));
    cond.appendChild(temp);

    for (int i = 0; i < attrInfos.length; ++i) {
      temp = doc.createElement("AttributeInfo");
      attrInfos[i].toXML(temp);
      cond.appendChild(temp);
    }
  }

  /**
    * Initializes this principal from the XML element <i>cond</i>.
    */
  private void fromXML(Element cond) throws ParseException {
    Node child = cond.getFirstChild();

    if (child == null) {
      throw new ParsingException();
    }

    Node temp = child.getFirstChild();

    if (temp == null) {
      throw new ParsingException();
    } else if ((constraint = temp.getNodeValue()) == null) {
      throw new ParsingException();
    }

    NodeList nl = cond.getElementsByTagName("AttributeInfo");
    int len = nl.getLength();

    attrInfos = new AttributeInfo[len];

    for (int i = 0; i < len; ++i) {
      attrInfos[i] = new AttributeInfo((Element) nl.item(i));
    } 
  }

  /**
    * Returns debugging info.
    */
  public String paramString() {
    StringBuffer sbuf = new StringBuffer();

    sbuf.append("[");

    sbuf.append("constraint");
    sbuf.append("=");
    sbuf.append("\"");
    sbuf.append(constraint);
    sbuf.append("\"");

    for (int i = 0; i < attrInfos.length; i++) {
      sbuf.append(",");
      sbuf.append(attrInfos[i].paramString());
    }

    sbuf.append("]");
    return sbuf.toString();
  }
}
