package lbl.akenti.certs.elements;

import java.io.IOException;
import java.text.ParseException;
import java.util.StringTokenizer;
import java.util.Vector;

import org.w3c.dom.Element;
import org.w3c.dom.Document;
import lbl.akenti.util.ParsingException;
import org.w3c.dom.NodeList;
import org.w3c.dom.Node;

import lbl.akenti.util.ParsingException;

/**
  * Actions that should be granted if the <i>Condition</i> supplied 
  * evaluates to <i>true</i>. If the condition evaluates to <i>false</i>
  * and the boolean flag <i>critical</i> is set to true, then access must
  * be denied.
  *
  * <pre>
  * ConditionalActions {
  *    Condition condition;
  *    vector&lt;string&gt; actions;
  *    boolean critical;
  * }
  * </pre>
  *
  * @see AttributeInfo
  *
  * @version 1.1 02/05/01
  *
  * @author Srilekha Mudumbai
  * @author Abdelilah Essiari
  **/

public final class ConditionalActions extends AkentiObject {

  private Condition condition;
  private String[] actions;
  private boolean critical;

  /**
    * Constructs this <i>ConditionalActions</i> using default values.
    */
  public ConditionalActions() {
    this (new Condition(), new String[] { "NO_RIGHTS" }, false); 
  }

  /**
    * Constructs this <i>ConditionalActions</i>.
    */
  public ConditionalActions(Condition condition, String[] actions) {
    this (condition, actions, false);
  }

  /**
    * Constructs this <i>ConditionalActions</i>.
    */
  public ConditionalActions(Condition condition,
			    String[] actions,
			    boolean critical) {
    if (condition == null) {
      throw new IllegalArgumentException();
    }

    this.condition = condition;

    if (actions == null) {
      this.actions = new String[0];
    } else {
      this.actions = actions;
    }

    this.critical = critical;
  }

  /**
    * Constructs this <i>ConditionalActions</i>.
    *
    * @param source stream used to initialize this object.
    */
  public
  ConditionalActions(AkentiInputStream source) throws IOException, ParseException {
    readObject(source);
  }

  /**
    * Constructs this <i>ConditionalActions</i>.
    *
    * @param element XML element used to parse this object.
    */
  public
  ConditionalActions(Element element) throws ParseException {
    fromXML(element);
  }

  /**
    * Returns the <i>Condition</i> that must evaluate to <i>true</i>
    * in order to grant the actions listed.
    *
    * @see Condition
    */
  public Condition getCondition() {
    return condition;
  }

  /**
    * Returns the booealn expression.
    *
    * @see Condition#getConstraint()
    */
  public String getConstraint() {
    return condition.getConstraint();
  }

  /**
    * Returns the <i>AttributeInfo</i>s.
    *
    * @see Condition#getAttributeInfos()
    */
  public AttributeInfo[] getAttributeInfos() {
    return condition.getAttributeInfos();
  }

  /**
    * Returns the actions.
    */
  public String[] getActions() {
    return actions;
  }

  /**
    * Returns <i>true</i> if the condition must be evaluate to <i>true</i>.
    */
  public boolean isCritical() {
    return critical;
  }

  /**
    * Writes this <i>ConditionalActions</i> to a <i>AkentiOutputStream</i>.
    */
  public void writeObject(AkentiOutputStream drain) throws IOException {
    condition.writeObject(drain);
    drain.write(' ');
    drain.write(actions.length);
   
    for (int i= 0; i < actions.length; ++i) {
      drain.write(' ');
      drain.write(actions[i]);
    }

    drain.write(' ');
    drain.write(critical);
  }

  /**
    * Initializes this <i>ConditionalActions</i> from a <i>AkentiInputStream</i>.
    */
  private  void 
  readObject(AkentiInputStream source) throws IOException, ParseException {
    condition = new Condition(source);

    int length = source.readInt();

    if (length < 0) {
      throw new ParsingException("no actions read"); 
    }

    actions = new String[length];

    for (int i=0; i < length; i++) {
      actions[i] = source.readString();
    }

    critical = source.readBoolean();
  }

  /**
    * Populates the XML element <i>condActions</i>.
    */
  public void toXML(Element condActions) {
    condActions.setAttribute("critical", critical? "true" : "false");

    Document doc = condActions.getOwnerDocument();
    Element temp = doc.createElement("Condition");

    condition.toXML(temp);
    condActions.appendChild(temp);

    if (actions.length != 0) {
      for (int i = 0; i < actions.length; ++i) {
        temp = doc.createElement("Action");
        temp.appendChild(doc.createTextNode(actions[i]));
        condActions.appendChild(temp);
      }
    }
  }

  /**
    * Initializes this principal from the XML element <i>condActions</i>.
    */
  private void fromXML(Element condActions) throws ParseException {
    String rep = condActions.getAttribute("critical");

    if (rep == null) {
      throw new ParsingException();
    }

    if (rep.equals("true")) {
      critical = true;
    } else if (rep.equals("false")) {
      critical = false;
    } else {
      throw new ParsingException();
    }

    Node child = condActions.getFirstChild();
  
    if (child == null) {
      throw new ParsingException();
    } else if (child.getNodeType() != Node.ELEMENT_NODE) {
      throw new ParsingException();
    }

    condition = new Condition((Element) child);

    NodeList list = condActions.getElementsByTagName("Action");
    int len = list.getLength();

    actions = new String[len];

    for (int i = 0; i < len; ++i) {
      Node temp = list.item(i).getFirstChild();
      
      if (temp == null) {
        throw new ParsingException();
      } else if ((actions[i] = temp.getNodeValue())  == null) {
        throw new ParsingException();
      }
    }
  }

  /**
    * Returns debugging info.
    */
  public String paramString() {
    return "ConditionalActions:paramString:to be implemented";
  }
}
