package lbl.akenti.certs.elements;

import java.security.Principal;
import java.util.Vector;
import java.util.StringTokenizer;
import java.io.IOException;
import java.text.ParseException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

import lbl.akenti.util.ParsingException;
import lbl.akenti.util.Pair;

/**
  * A Distinguished Name is a multi-component name used in X509
  * certifificates to uniquely identify an entity. 
  *
  * Examples of attributes are listed below.
  *
  * <pre>
  *   commonName                   "CN"
  *   user ID                      "UID"
  *   countryName                  "C"
  *   localityName                 "L"
  *   Email Address                "E"
  *   stateOrProvinceName          "ST"
  *   organizationName             "O"
  *   organizationalUnitName       "OU"
  * </pre>
  *
  * Some of the attributes may have more than one value. And we allow user 
  * defined attributes. 
  *
  * <pre>
  *    We also handle two formats:
  *   
  *    <i>X500</i> it starts with the most general components 
  *                and uses a forward slash as a separator.
  *                e.g. /C=US/O=LBL/OU=DSD/CN=Ernest Lawrence
  *
  *    <i>X400</i> is the reverse of the X500 and uses
  *                a comma as a separator.
  *                e.g. CN=Ernest Lawrence,OU=DSD,O=LBL,C=US
  *
  * </pre>
  *
  * @version 1.1 00/05/02
  * @author Srilekha Mudumbai
  * @author Abdelilah Essiari
  */

public final class DistinguishedName extends AkentiObject {

  private Vector pairs;

  /**
    * Constructs this object. 
    */
  public DistinguishedName() {
    pairs = new Vector();
    pairs.add(new Pair("CN", "NO_NAME"));
  }

  /**
    * Constructs this object.
    *
    * @param dn  string assumed to be in X500 format 
    */
  public DistinguishedName(String dn) throws ParsingException {
    pairs = new Vector();
    parse500(dn);
  }

  /**
    * Constructs this object.
    *
    * @param dn Principal which uses the X400 format 
    */
  public DistinguishedName(Principal dn) throws ParsingException {
    // XXX
    StringTokenizer st = new StringTokenizer(dn.getName(), ",");
    StringBuffer sb =  new StringBuffer();

    do {
      sb.insert(0, '/');
      sb.insert(1, st.nextToken().trim());
    } while (st.hasMoreTokens());

    pairs = new Vector();
    parse500(sb.toString());
  }

  /**
    * Constructs this object.
    *
    * @param source AkentiInputStream used to read this object 
    *
    * @see readObject(AkentiInputStream)
    */
  public 
  DistinguishedName(AkentiInputStream source) throws IOException, ParseException {
    readObject(source);
  }

  /**
    * Constructs this object.
    *
    * @param element Element used to read this object from XML.
    *
    * @see fromXML(Element)
    */
  public DistinguishedName(Element element) throws ParseException {
    fromXML(element);
  }

  /**
    * Parses the X500 representation.
    */
  private void parse500(String dn) throws ParsingException {
    StringTokenizer st = new StringTokenizer(dn, "/=", true);

    do {
      try {
        if (!st.nextToken().equals("/")) {
          throw new ParsingException("DistinguishedName:expecting (/)");
        }

        String attribute = st.nextToken();

        if (!st.nextToken().equals("=")) {
          throw new ParsingException("DistinguishedName:expecting (=)");
        }

        // The following checks are sufficient for now. :)
        String value = st.nextToken();

        switch(value.charAt(0)) {
          case '"':
          case '=':
          case '/':
          case ' ':
            throw new ParsingException("DistinguishedName:bad value");
        }

        if (value.charAt(value.length() - 1) == ' ') {
          throw new ParsingException("DistinguishedName:bad value");
        }

        pairs.add(new Pair(attribute, value));
      } catch(java.util.NoSuchElementException ex) {
        throw new ParsingException("DistinguishedName:missing token");
      }
    } while (st.hasMoreTokens());
  }

  /**
    * Returns the first value associated with <i>attribute</i>. 
    */
  public String getValue(String attribute) {
    for (int i=0; i < pairs.size(); i++) {
      Pair pair = (Pair) pairs.elementAt(i);

      if (pair.containsAttribute(attribute)) {
        return pair.getValue();
      }
    }
    
    return null;
  }

  /**
    * Returns all values associated with <i>attribute</i>. 
    */
  public String[] getValues(String attribute) {
    Vector values = new Vector();

    for (int i=0; i < pairs.size(); i++) {
      Pair pair = (Pair) pairs.elementAt(i);

      if (pair.containsAttribute(attribute)) {
        values.add(pair.getValue());
      }
    }

    String[] ret = new String[values.size()];

    values.toArray(ret);
    return ret;
  }

  /**
    * Returns <i>true</i> if this object contains the  attribute/value pair
    * specified by the arguments.
    */
  public boolean containsAttributeValue(String attr, String value) {
    return pairs.contains(new Pair(attr,value));
  }

  /**
    * Returns <i>true</i> if there is at least one value associated
    * with <i>attribute<i>.
    */
  public boolean containsAttribute(String attribute) {
    return numOfOccurrences(attribute) != 0; 
  }

  /**
    * Returns <i>true</i> if object contains an attribute that
    * corresponds to <i>value</i>.
    */
  public boolean containsValue(String value) {
    boolean found = false;

    for (int i=0; !found && i < pairs.size(); i++) {
      Pair pair = (Pair) pairs.elementAt(i);

      found = pair.containsValue(value);
    }

    return found;
  }

  /**
    * Returns the number of values associated with <i>attribute</i>.
    */
  public int numOfOccurrences(String attribute) {
    int count = 0;

    for (int i=0; i < pairs.size(); i++) {
      Pair pair = (Pair) pairs.elementAt(i);

      if (attribute.equalsIgnoreCase(pair.getAttribute())) {
        ++count;
      }
    }
   
    return count; 
  }

  /**
    * Returns the number of attribute/value pairs.
    */
  public int size() {
    return pairs.size();
  }

  /**
    * Returns true if <i>index</i> than the number  of attribute/value
    * pairs.
    */
  public boolean hasMorePairs(int index) {
    return index < size();
  }

  /**
    */
  public Pair getPairAt(int index) {
    return (Pair) pairs.elementAt(index);
  }

  /**
    * Writes this object to an <i>AkentiOutputStream</i>.
    */
  public void 
  writeObject(AkentiOutputStream drain) throws IOException {
    drain.write(to500String());
  }

  /**
    * Initailizes this object from an <i>AkentiInputStream</i>.
    */
  private void 
  readObject(AkentiInputStream source) throws IOException, ParseException {
    pairs = new Vector();
    parse500(source.readString());
  }

  /**
    * Populates the XML element <i>dn</i>.
    */
  public void toXML(Element dn) {
    Document doc = dn.getOwnerDocument();
    
    dn.appendChild(doc.createTextNode(to500String()));
  }

  /**
    * Initializes this object from the XML element <i>dn</i>.
    */
  private void fromXML(Element dn) throws ParseException {
    Node child = dn.getFirstChild();

    if (child == null) {
      throw new ParsingException();
    }

    String rep = child.getNodeValue();

    if (rep == null) {
      throw new ParsingException();
    }

    pairs = new Vector();
    parse500(rep);
  }

  /**
    * Returns the X400 format of this <i>DistinguishedName.
    *
    */
  public String to400String() {
    StringBuffer sbuf = new StringBuffer();

    for (int i = pairs.size() - 1; i >= 1; i--) {
      Pair pair = (Pair) pairs.elementAt(i);

      sbuf.append(pair.getAttribute());
      sbuf.append('=');
      sbuf.append(pair.getValue());
      sbuf.append(',');
    }

    Pair pair = (Pair) pairs.elementAt(0);

    sbuf.append(pair.getAttribute());
    sbuf.append('=');
    sbuf.append(pair.getValue());
    return sbuf.toString();
  }

  /**
    * Returns the X400 format of this <i>DistinguishedName.
    */
  public String to500String() {
    StringBuffer sbuf = new StringBuffer();

    for (int i=0; i < pairs.size(); i++) {
      Pair pair = (Pair) pairs.elementAt(i);

      sbuf.append('/');
      sbuf.append(pair.getAttribute());
      sbuf.append('=');
      sbuf.append(pair.getValue());
    }

    return sbuf.toString();
  }

  /**
    * Tests for equality. 
    *
    * Note this is useful for containers such as vectors or sets.
    */
  public boolean equals(Object obj) {
    if (obj instanceof DistinguishedName) {
      DistinguishedName dname = (DistinguishedName) obj;
      
      return pairs.equals(dname.pairs);
    }

    return false;
  }

  /**
    * Computes a hash.
    *
    * Note this is useful for containers such as hashsets and hastables.
    */
  public int hashCode() {
    return toString().hashCode();
  }

  /**
    * Returns a string representation of this object that can be
    * used by gui object. 
    */
  public String toString() {
    String rep = getValue("CN");

    return rep == null? to400String() : rep;
  }

  /**
    * Used for debugging.
    */
  public String paramString() {
    StringBuffer sbuf = new StringBuffer();

    sbuf.append("[");
    sbuf.append(pairs.elementAt(0));

    for (int i=1; i < pairs.size(); i++) {
      sbuf.append(",");
      sbuf.append(pairs.elementAt(i));
    }

    sbuf.append("]");
    return sbuf.toString();
  }
}
