package lbl.akenti.dirServices;

import java.net.URL;
import java.net.MalformedURLException;
import java.util.Map;
import java.util.ArrayList;
import java.io.IOException;
import java.io.FilenameFilter;
import java.io.File;
import java.text.ParseException;
import java.security.cert.X509Certificate;
import java.security.cert.CertificateException;
import lbl.akenti.certs.AkentiCertificateFactory;
import lbl.akenti.certs.AkentiCertificate;
import lbl.akenti.util.X509CertReader;

/**
  * Class <code>FileCertificateRetrieval</code> loads certificates from 
  * file system directories.
  * 
  * @author  Abdelilah Essiari
  * @see     lbl.akenti.certs.AkentiCertificate;
  * @see     lbl.akenti.certs.AkentiCertificateFactory;
  * @see     lbl.akenti.util.X509CertReader;
  */
public 
class FileCertificateRetrieval extends CertificateRetrieval {

  /**
    * Construct this <code>CertificateRetrieval</code>.
    */ 
  public FileCertificateRetrieval() {
  }

  /**
    * Creates a simple file name filter.
    */
  private FilenameFilter createFilenameFilter(final String hash) {
    return (hash == null) ? 
              null :
              new FilenameFilter() {
                public boolean accept(File dir, String name) {
                  // TODO
                  return true;
                }
              };
  }

  protected String[]
  listFiles(URL url) {
    return new File(url.getFile()).list();
  }

  private String[]
  listFiles(URL url, String hash) {
    String[] names = listFiles(url);
   
    if (names == null || hash == null) {
      return names;
    }

    ArrayList v = new ArrayList();
    FilenameFilter filter = createFilenameFilter(hash);

    for (int i = 0 ; i < names.length ; i++) {
      if (filter.accept(null, names[i])) {
        v.add(names[i]);
      }
    }

    return (String[]) (v.toArray(new String[0]));
  }

  /**
    * Retrieves <code>X509Certificate</code>s.
    *
    * <P>
    *     Assumes <i>url</i> points to a directory.
    *     Assumes encType is <i>PEM</i>.
    * </P>
    */
  final public X509Certificate[] 
  retrieveX509Certificates(URL url, Map args) {
    String[] names = listFiles(url, (String) args.get("hash"));

    if (names == null) {
      return new X509Certificate[0];
    }

    ArrayList v = new ArrayList();
    X509CertReader xcr = new X509CertReader();

    for (int i = 0; i < names.length; ++i) {
      try { 
        String name = url.getFile() + File.separator + names[i];
        URL temp = new URL(url.getProtocol(), url.getHost(), url.getPort(), name);
        X509Certificate[] certs = xcr.readX509Certs(temp);
  
        for (int j = 0; j < certs.length; ++j) { 
          v.add(certs[j]);
        }
      } catch(MalformedURLException ex) {
        throw new RuntimeException(ex);
      } catch(IOException ex) {
        // DO NOTHING
      } catch(CertificateException ex) {
        // DO NOTHING
      }
    }

    return (X509Certificate[]) v.toArray(new X509Certificate[0]);
  }

  /**
    * Retrieves <code>AkentiCertificate<</code>s.
    *
    * <P>
    *     Assumes url points to a directory.
    *     Default encoding type is <i>PEM</i>.
    * </P>
    */
  final public AkentiCertificate[]
  retrieveAkentiCertificates(URL url, Map args) { 
    String[] names = listFiles(url, (String) args.get("hash"));

    if (names == null) {
      return new AkentiCertificate[0];
    }

    String encType = (String) args.get("encType"); 
    
    if (encType == null) {
      encType = "PEM";
    }

    ArrayList v = new ArrayList(names.length);
    AkentiCertificateFactory acf = AkentiCertificateFactory.getInstance();

    for (int i = 0; i < names.length; ++i) {
      try {
        String name = url.getFile() + File.separator + names[i];
        URL temp = new URL(url.getProtocol(), url.getHost(), url.getPort(), name);
        AkentiCertificate cert = acf.initCertFromURL(encType, temp);

        v.add(cert);
      } catch(MalformedURLException ex) {
        throw new RuntimeException(ex);
      } catch(IOException ex) {
        // DO NOTHING
      } catch(ParseException ex) {
        // DO NOTHING
      }
    }

    return (AkentiCertificate []) v.toArray(new AkentiCertificate[0]);
  }
}
