/**
  * A simple TLS echo server.
  * 
  * Assumes Java 1.4. In java 1.2 and 1.3 the net.ssl pakage was not
  * part of the standard distribution. Some of the classes were under
  * com/sun/net/ssl instead of javax/net/ssl. There was also a bug in
  * its handling of p12 files created by openSSL. It did handle p12
  * file created by Netscape correctly.
  * 
  * This server has a p12 file that stores its private key and its 
  * chain of certificates.
  *
  * @author Abdelilah Essiari
  */

package lbl.akenti.examples.SSLServer;

import java.security.cert.X509Certificate;
import java.security.cert.Certificate;
import java.security.cert.CertificateException;
import java.security.cert.CertificateFactory;
import java.security.KeyStore;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.UnrecoverableKeyException;
import java.security.KeyManagementException;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.BufferedInputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;

import javax.net.ssl.KeyManagerFactory;
import javax.net.ssl.TrustManagerFactory;
import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLServerSocket;
import javax.net.ssl.SSLSocket;
import javax.net.ssl.SSLSession;
import javax.net.ServerSocketFactory;
import javax.net.ssl.SSLPeerUnverifiedException;


import java.net.Socket;

public class TestServer {
    /**
      * Echo server
      *
      * Loads p12 file into key store.
      * Loads trusted cas into key store.
      * Intializes the context.
      * Accpets connections
      * Dispatches a thread to do the reading and the echoing
      * 
      */
    public static void main(String args[]) {
      try {
        /**
          * <i>p12FileName</i> holds client's key and chain.
          * <i>casFileName</i> holds tusted cas' x509 certs.
          * <i>password</i> locks the p12 file.
          * <i>port</i> at which the server is listening. 
          *
          */
        String p12FileName = "server.p12";
        String casFileName = "cas.pem";
        char[] password = "testing".toCharArray();
        int port = 9999;
    
        KeyStore ks = KeyStore.getInstance("pkcs12");

        // Loads p12 file
        ks.load(new FileInputStream(p12FileName), password);

        // Adds trusted ca certs to key store.
        // It is important to use a stream that supports marks.
        CertificateFactory cfac = CertificateFactory.getInstance("X.509");
        BufferedInputStream bis = new BufferedInputStream(
          new FileInputStream(casFileName)
        );

        do {
          // We use th dn for an alias 
          X509Certificate cert = (X509Certificate) cfac.generateCertificate(bis);
          String alias = cert.getSubjectDN().toString();

          ks.setCertificateEntry(alias, cert);
        } while (bis.available() > 0);

        KeyManagerFactory kfac = KeyManagerFactory.getInstance("SunX509");

        kfac.init(ks, password);

        TrustManagerFactory tfac = TrustManagerFactory.getInstance("SunX509");

        tfac.init(ks);
       
        // We use TLS.
        SSLContext ctx = SSLContext.getInstance("TLS");

        // Initialize the context.
        // U can always save the random number
        // get things going faster.
       
        System.out.println("this will take some time...");

        ctx.init(
          kfac.getKeyManagers(),   // Used to authenticate to client
          tfac.getTrustManagers(), // Used to authenticate the client
          null                     // here u could generate a random 
        );

        // Get a factory that knows how to create SSL sockets.
        ServerSocketFactory fac = ctx.getServerSocketFactory();

        SSLServerSocket serverSocket = (SSLServerSocket) fac.createServerSocket(port);

	//  setWant allows for an optional client certificate
	// serverSocket.setWantClientAuth(true);

	// The setNeed requires a client certificate
	serverSocket.setNeedClientAuth(true);

        // From here on we do the usual stuff
        // Hard to tell we are using an ssl socket
        // but we are.

        while (true) {
          System.out.println("accepting ...");

          final SSLSocket sock = (SSLSocket) serverSocket.accept();
	  SSLSession session = sock.getSession();
	  try {
	    Certificate clientCert[] =  session.getPeerCertificates();
	    for ( int i=0; i< clientCert.length; i ++) {
	      System.out.println(((X509Certificate)clientCert[i]).getSubjectDN().toString());
	    }

	  } catch (SSLPeerUnverifiedException e) {
	      System.out.println ("Client certificate did not verify");
	  }

           
          new Thread() {
            public void run() {
              try {
                DataInputStream in = new DataInputStream(sock.getInputStream());
                int len = in.readInt();
                
                if (len < 0 || len > 100) {
                  System.out.println("Len = " + len);
                  return;
                }

                byte[] buffer = new byte[len];

                in.readFully(buffer, 0, len);
                
                String str = new String(buffer);
                DataOutputStream out = new DataOutputStream(sock.getOutputStream());

		System.out.println("Got message: " + str);
                out.writeInt(str.length());
                out.writeBytes(str);
                out.flush();
              } catch(IOException ex) {
                System.out.println(ex);
              } finally {
                try { 
                  sock.close(); 
                } catch(IOException ex) { 
                  System.out.println(ex);
                }
              }
            }
          }.start();
        }
      } catch (KeyManagementException  e) {
        e.printStackTrace();
      } catch (UnrecoverableKeyException e) {
        e.printStackTrace();
      } catch (NoSuchAlgorithmException e) {
        e.printStackTrace();   
      } catch (KeyStoreException e) {
        e.printStackTrace();
      } catch (FileNotFoundException e) {
        e.printStackTrace(); 
      } catch (IOException e) {
        e.printStackTrace();
      } catch (CertificateException e) {
        e.printStackTrace();
      }
    }
}
