package lbl.akenti.gen.controller.pol;

import java.util.HashSet;
import java.util.Properties;
import java.net.URL;
import java.security.Security;

import lbl.akenti.gen.controller.LBLFrame;
import lbl.akenti.gen.controller.LBLResourceFrame;
import lbl.akenti.gen.controller.LBLIssuerFrame;
import lbl.akenti.gen.controller.LBLTimeFrame;
import lbl.akenti.gen.controller.LBLCacheFrame;
import lbl.akenti.gen.controller.LBLReviewFrame;
import lbl.akenti.gen.controller.LBLSaveFrame;
import lbl.akenti.gen.controller.AbstractController;

import lbl.akenti.util.UTCTime;

import lbl.akenti.gen.common.GenInfo;

import lbl.akenti.certs.PolicyCertGenerator;
import lbl.akenti.certs.elements.CertificateAuthorityInfo;
import lbl.akenti.certs.elements.Directory;
import lbl.akenti.certs.elements.UseCondIssuerGroup;
import lbl.akenti.certs.elements.AkentiIssuer;
import lbl.akenti.certs.elements.AkentiPrincipal;

/**
  * Generates <i>Policy</i> certificates. 
  *
  * @see AbstractController 
  */
public class PolController 
  extends AbstractController 
  implements PolConsts {
 
  /**
    * Constructor. 
    * 
    * @see AbststactController#doGetFrameAt(int)
    */
  public PolController() {
    // We will be using 9 frames.
    super(9);
  }

  /** 
    * Shows the first frame.
    *
    * @see AbstractController#start()
    */
  public void start() {
    super.start();
  }

  /** 
    * Creates the frame associated with the given index.
    *
    * @param index of the frame to be created 
    * @return an LBLFrame that corresponds to index.
    *
    * @see AbstractController#getFrameAt(index)
    */
  protected LBLFrame doGetFrameAt(int index) {
    switch(index) {
      case RESOURCE_INDEX:
        return new LBLResourceFrame(TITLES[index], HEADERS[index]);
      case CA_INFOS_INDEX:
        return new PolCAInfosFrame(TITLES[index], HEADERS[index]);
      case UC_GROUPS_INDEX:
        return new PolUCGroupsFrame(TITLES[index], HEADERS[index]);
      case ISSUER_INDEX:
        return new LBLIssuerFrame(TITLES[index], HEADERS[index]);
      case ATTR_DIRS_INDEX:
        return new PolAttrDirsFrame(TITLES[index], HEADERS[index]);
      case TIME_INDEX:  
        return new LBLTimeFrame(TITLES[index], HEADERS[index]);
      case CACHE_INDEX:
        return new LBLCacheFrame(TITLES[index], HEADERS[index]);
      case REVIEW_INDEX:
        return new LBLReviewFrame(TITLES[index], HEADERS[index]);
      case SIGN_INDEX:
        return new LBLSaveFrame(TITLES[index], HEADERS[index]);
      default:
        throw new RuntimeException();
    }
  }

  /** 
    * Loads inputs in the frame that corresponds to <i>index</i>.
    *
    * @param index index of frame that needs to be loaded
    *
    * @return true if this method should be called when displaying this
    *              frame in the future.
    *         false otherwise.
    *
    * @see AbstractController#showNext()
    * @see AbstractController#loadInputsIntoFrameAt(int)
    * @see AbstractController#clearBits()
    */
  public boolean loadInputsIntoFrameAt(int index) throws Exception {
    switch(index) {
      case CA_INFOS_INDEX: 
        LBLResourceFrame resourceFrame = (LBLResourceFrame) getFrameAt(RESOURCE_INDEX);
        URL base = new URL(resourceFrame.getBaseURL());
        String resource = resourceFrame.getResource();

        if (info == null || !(info.hasResource(resource) && info.hasBaseURL(base))) {
          // Make sure we load the GenInfo first.
          GenInfo tempInfo = new GenInfo(resource, base);

          tempInfo.loadResFileInfo();
          tempInfo.loadPolicies();

          
          this.info = tempInfo;

          PolCAInfosFrame caInfosFrame = (PolCAInfosFrame) getFrameAt(index);

          caInfosFrame.setOptional(!info.isRoot());
          caInfosFrame.setCAInfos(info.getCertificateAuthorityInfos());
          clearBits();
        } 

        return true;
      case UC_GROUPS_INDEX: 
        PolUCGroupsFrame ucGroupFrame = (PolUCGroupsFrame) getFrameAt(index);

        ucGroupFrame.setUCGroups(info.getUseCondIssuerGroups());
        return true;
      case ISSUER_INDEX: 
        LBLIssuerFrame issuerFrame = (LBLIssuerFrame) getFrameAt(index);

        ucGroupFrame = (PolUCGroupsFrame) getFrameAt(UC_GROUPS_INDEX);
        UseCondIssuerGroup[] ucGroups = ucGroupFrame.getUCGroups();
        HashSet ucIssuers = new HashSet();

        for (int i = 0; i < ucGroups.length; ++i) {
          AkentiPrincipal[] aps = ucGroups[i].getUCIssuers();

          for (int j = 0; j < aps.length; ++j) {
            ucIssuers.add(aps[j]);
          }
        }
        
        AkentiPrincipal []temp = new AkentiPrincipal[ucIssuers.size()];
         
        ucIssuers.toArray(temp);
        issuerFrame.setIssuers(temp);
        return false;
      case ATTR_DIRS_INDEX:
        PolAttrDirsFrame attrDirsFrame = (PolAttrDirsFrame) getFrameAt(index);

        attrDirsFrame.clear();
        attrDirsFrame.addAttrDirs(info.getAttributeDirectories());
        return true;
      case TIME_INDEX: 
        return true;
      case CACHE_INDEX: 
        LBLCacheFrame cacheFrame = (LBLCacheFrame) getFrameAt(CACHE_INDEX);

        cacheFrame.setCacheTime(info.getCacheTime());
        return true;
      case REVIEW_INDEX: 
        PolicyCertGenerator gen = new PolicyCertGenerator();

        gen.setVersionNumber(2);
        gen.setUniqueID("UID");
        
        UTCTime utc = new UTCTime();

        LBLTimeFrame timeFrame = (LBLTimeFrame) getFrameAt(TIME_INDEX);

        gen.setNotBefore(utc);
        gen.setNotAfter(new UTCTime(utc, timeFrame.getLifeTime()));

        issuerFrame = (LBLIssuerFrame) getFrameAt(ISSUER_INDEX);
        gen.setIssuer(new AkentiIssuer(issuerFrame.getIssuer()));

        PolCAInfosFrame caInfosFrame = (PolCAInfosFrame) getFrameAt(CA_INFOS_INDEX);

        CertificateAuthorityInfo []infos = caInfosFrame.getCAInfos();
        
        if (infos.length != 0) {
          gen.setCertificateAuthorityInfos(infos);
        }

        resourceFrame = (LBLResourceFrame) getFrameAt(RESOURCE_INDEX);
        gen.setResource(resourceFrame.getResource());

        ucGroupFrame = (PolUCGroupsFrame) getFrameAt(UC_GROUPS_INDEX);
        gen.setUseCondIssuerGroups(ucGroupFrame.getUCGroups());

        attrDirsFrame = (PolAttrDirsFrame) getFrameAt(ATTR_DIRS_INDEX);
       
        Directory [] dirs = attrDirsFrame.getAttrDirs();

        if (dirs.length != 0) {
          gen.setAttributeDirectories(dirs);
        }
        
        cacheFrame = (LBLCacheFrame) getFrameAt(CACHE_INDEX);
        long cacheTime = cacheFrame.getCacheTime();

        if (cacheTime != -1) {
          gen.setCacheTime(cacheTime);
        }

        LBLReviewFrame reviewFrame = (LBLReviewFrame) getFrameAt(REVIEW_INDEX);

        reviewFrame.setCertificate(gen.generate());
        this.gen = gen;
        return false;
      case SIGN_INDEX: 
        LBLSaveFrame saveFrame = (LBLSaveFrame) getFrameAt(SIGN_INDEX);

        saveFrame.clear();
        return false;
      default:
        throw new RuntimeException();
    }
  }

  /**
    */
  protected String getHelpMessage(int index) {
    return helpMess[index];
  }
 
  /**
    */
  public static void main (String[] args) {
    try {
      // Load Properties.
      if (args.length == 1) {
        Properties props = System.getProperties();
        
        props.load(new java.io.FileInputStream(args[0]));
        System.out.println(props);
      }
       
      Security.addProvider(new com.sun.net.ssl.internal.ssl.Provider());

      new PolController().start();
    } catch(Exception e) {
      e.printStackTrace();
    }
  }
}
