package lbl.akenti.gen.controller.uc;

import com.sun.net.ssl.internal.ssl.Provider;
import java.io.FileInputStream;
import java.net.URL;
import java.security.Security;
import java.util.Properties;
import javax.swing.UIManager;
import lbl.akenti.certs.UseCondCertGenerator;
import lbl.akenti.certs.elements.AkentiIssuer;
import lbl.akenti.certs.elements.Condition;
import lbl.akenti.gen.common.GenInfo;
import lbl.akenti.gen.controller.AbstractController;
import lbl.akenti.gen.controller.LBLFrame;
import lbl.akenti.gen.controller.LBLIssuerFrame;
import lbl.akenti.gen.controller.LBLResourceFrame;
import lbl.akenti.gen.controller.LBLReviewFrame;
import lbl.akenti.gen.controller.LBLSaveFrame;
import lbl.akenti.gen.controller.LBLTimeFrame;
import lbl.akenti.util.UTCTime;

/**
  * Generates a Use Condition Certificate.
  *
  * @see AbstractController
  */
public class UCController 
  extends AbstractController 
  implements UCConsts {

  /**
    * Constructor:  
    */
  public UCController() {
    // We will be using 7 frames.
    super(7);
  }

  /**
    * Shows the first frame.
    *
    * @see AbstractController#start()
    */
  public void start() {
    super.start();
  }

  /**
    * Creates the frame associated with the given index.
    *
    * @param index of the frame to be created
    * @return an LBLFrame that corresponds to index.
    *
    * @see AbstractController#getFrameAt(index)
    */
  protected LBLFrame doGetFrameAt(int index) {
    switch(index) {
      case 0:
	return new LBLResourceFrame(TITLES[RESOURCE_INDEX], HEADERS[RESOURCE_INDEX]); 
      case 1:
	return new LBLIssuerFrame(TITLES[ISSUER_INDEX], HEADERS[ISSUER_INDEX]);
      case 2:
	return new UCExpressionFrame(TITLES[EXPR_INDEX], HEADERS[EXPR_INDEX]);
      case 3:
        return new UCScopeFrame(TITLES[SCOPE_INDEX], HEADERS[SCOPE_INDEX]);
      case 4:
	return new LBLTimeFrame(TITLES[TIME_INDEX], HEADERS[TIME_INDEX]);
      case 5:
	return new LBLReviewFrame(TITLES[REVIEW_INDEX], HEADERS[REVIEW_INDEX]);
      case 6:
	return new LBLSaveFrame(TITLES[SIGN_INDEX], HEADERS[SIGN_INDEX]);
      default:
	throw new RuntimeException();
    }
  }
    
  /**
    * Loads inputs in the frame that corresponds to <i>index</i>.
    *
    * @param index index of frame that needs to be loaded
    *
    * @return true if this method should be called when displaying this
    *              frame in the future.
    *         false otherwise.
    *
    * @see AbstractController#showNext()
    * @see AbstractController#loadInputsIntoFrameAt(int)
    * @see AbstractController#clearBits()
    */
  protected boolean  
  loadInputsIntoFrameAt(int nextFrameIndex) throws Exception {
    switch(nextFrameIndex) {
      case 1:
        LBLResourceFrame resourceFrame = (LBLResourceFrame) getFrameAt(RESOURCE_INDEX);
        URL base = new URL(resourceFrame.getBaseURL());
        String resource = resourceFrame.getResource();

        if (info == null || !(info.hasResource(resource) && info.hasBaseURL(base))) { 
          // Make sure we load the GenInfo first.
          GenInfo tempInfo = new GenInfo(resource, base);

          tempInfo.loadResFileInfo();
          tempInfo.loadPolicies();

          if (!tempInfo.getTopResource().equals(resource)) {
            throw new Exception("No policy cert corresponding to " + resource);
          }

          this.info = tempInfo;

	  LBLIssuerFrame issuerFrame = (LBLIssuerFrame) getFrameAt(ISSUER_INDEX);

          issuerFrame.clear();
	  issuerFrame.setIssuers(info.getUCIssuers());
	  clearBits();
        }

        return true;
      case 2:
        UCExpressionFrame expFrame = (UCExpressionFrame) getFrameAt(EXPR_INDEX);
	    
        expFrame.clear();
        expFrame.setHintAttributes(info.getGenAttributes());
        return true;
      case 3: 
        UCScopeFrame scopeFrame = (UCScopeFrame) getFrameAt(SCOPE_INDEX);

        scopeFrame.clear();
        scopeFrame.setActions(info.getActions());
        return true;
      case 4:
        return true;
      case 5:
        UseCondCertGenerator gen = new UseCondCertGenerator();

        gen.setVersionNumber(2);
        gen.setUniqueID("UID");

        UTCTime utc = new UTCTime();

        LBLTimeFrame timeFrame = (LBLTimeFrame) getFrameAt(TIME_INDEX);

        gen.setNotBefore(utc);
        gen.setNotAfter(new UTCTime(utc, timeFrame.getLifeTime()));

        LBLIssuerFrame issuerFrame = (LBLIssuerFrame) getFrameAt(ISSUER_INDEX);

        gen.setIssuer(new AkentiIssuer(issuerFrame.getIssuer()));

        resourceFrame = (LBLResourceFrame) getFrameAt(RESOURCE_INDEX);
        gen.setResource(resourceFrame.getResource());

        expFrame = (UCExpressionFrame) getFrameAt(EXPR_INDEX);
        scopeFrame = (UCScopeFrame) getFrameAt(SCOPE_INDEX);
        gen.setLocalScope(scopeFrame.isScopeLocal());
        gen.setCritical(scopeFrame.getCritical());

        Condition cond = new Condition(
          expFrame.getExpression(),
          expFrame.getAttributeInfos()
        );
           
        gen.setCondition(cond);
        gen.setActions(scopeFrame.getActions());

        LBLReviewFrame reviewFrame = (LBLReviewFrame) getFrameAt(REVIEW_INDEX);

        reviewFrame.setCertificate(gen.generate());
        this.gen = gen;
        return false;
      case 6:
        return false;
      default:
        throw new RuntimeException();
    }
  }
    
  /**
    */
  protected String getHelpMessage(int currentFrameIndex) {
    return helpMessages[currentFrameIndex];
  }

  /**
    */
  public static void main (String[] args) {
    try {
      // Load Properties.
      if (args.length == 1) {
        Properties props = System.getProperties();

        props.load(new FileInputStream(args[0]));
        // System.out.println(props);
   
        String temp = props.getProperty("os.name");

        if (!temp.equals("SunOS")) {
          UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
        }
      }

      Security.addProvider(new Provider());

      new UCController().start();
    } catch(Exception e) {
      e.printStackTrace();
    }
  }
}
