package lbl.akenti.messaging;

import java.io.Writer;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.IOException;
import java.util.List;
import java.util.Collections;
import java.util.ArrayList;
import java.util.Iterator;
import java.text.ParseException;
import org.w3c.dom.Element;

import javax.xml.bind.JAXBException;
import lbl.akenti.jaxb.AkentiRequestsGenerated;
import lbl.akenti.jaxb.AkentiRequestType;
import lbl.akenti.jaxb.CheckAccessRequestType;
import lbl.akenti.jaxb.CheckPolicyRequestType;
import lbl.akenti.jaxb.ResourceBrowseRequestType;
import lbl.akenti.jaxb.CreateResourceRequestType;
import lbl.akenti.jaxb.UploadPolicyCertificateRequestType;
import lbl.akenti.jaxb.GetResourceInfoRequestType;

/**
  * Represents an immutable non-empty list of <i>AkentiRequest</i>s that
  * are handled by the Akenti server.                            <BR>
  *
  * Currently The server handles the following requests:         <BR>
  *
  * The server handles the following requests:                   <BR>
  *
  * CheckAccessRequest:                                          <BR>
  *   Used when determining a user's access to a resource.       <BR>
  *
  * CheckPolicyRequest:                                          <BR>
  *   Used by stakholders to debug policy                        <BR>
  *
  * ResourceBrowseRequest:                                       <BR>
  *   Used to determine daughter resources of given resource     <BR>
  *
  * CreateResourceRequest:                                       <BR>
  *   Used to create resources                                   <BR>
  *
  * UploadPolicyCertificateRequest:                              <BR>
  *   Used to Upload a policy certificate                        <BR>
  *
  * GetResoourceInfoRequest:                                     <BR>
  *   Used to upload a resource attrs/hints                      <BR>
  *
  * Each request contains a client-generated unique id that can be useful 
  * when clients are sending or keeping track of multiple requests.
  * 
  * @author Abdelilah Essiari.
  */

final public 
class AkentiRequests {
  private List requests;

  /**
    * Constructs this <i>AkentiRequests</i>.
    *
    * @param ar a single AkentiRequest.
    */
  public AkentiRequests(AkentiRequest ar) {
    this(new AkentiRequest[]{ar});
  }

  /**
    * Constructs this <i>AkentiRequests</i>.
    *
    * @param ars an array of AkentiRequest of size at least 1.
    */
  public AkentiRequests(AkentiRequest[] ars) {
    requests = new ArrayList(ars.length);
    requests.add(ars[0]);
   
    for (int i = 1; i < ars.length; ++i) {
      requests.add(ars[i]);
    } 
    
    requests = Collections.unmodifiableList(requests);
  }

  /**
    * Constructs this <i>AkentiRequests</i> from an input stream
    * containing <i>XML</i> data.
    *
    * @throws IOException when unable to read this object 
    */
  public AkentiRequests(InputStream in) throws IOException, ParseException  {
    try {
      init((AkentiRequestsGenerated) Util.getUnmarshaller().unmarshal(in));
    } catch(JAXBException ex) {
      throw new IOException(ex.getMessage());
    }
  }

  /**
    * Constructs this <i>AkentiRequests</i> from a <i>DOM</i> element.
    *
    * @throws IOException when unable to build this object 
    */
  public AkentiRequests(Element element) throws IOException, ParseException {
    try {
      init((AkentiRequestsGenerated) Util.getUnmarshaller().unmarshal(element));
    } catch(JAXBException ex) {
      throw new ParseException(ex.getMessage(), -1);
    }
  }

  /**
    * Returns the requests as an iterator.
    *
    * @see AkentiRequest
    */
  public Iterator getAkentiRequests() {
    return requests.listIterator();
  }
  
  /**
    * Returns the requests as an array.
    *
    * @see AkentiRequest
    */
  public AkentiRequest[] toAkentiRequestsArray() {
    AkentiRequest[] ret = new AkentiRequest[requests.size()];

    requests.toArray(ret);
    return ret;
  }

  /**
    */
  public void toStream(OutputStream out) throws IOException {
    try {
      Util.getMarshaller().marshal(toJAXBImpl(), out);
    } catch(JAXBException ex) {
      throw new IOException(ex.getMessage());
    }
  }

  /**
    */
  public void toWriter(Writer writer) throws IOException {
    try {
      Util.getMarshaller().marshal(toJAXBImpl(), writer);
    } catch(JAXBException ex) {
      throw new IOException(ex.getMessage());
    }
  }

  /**
    */
  public void toElement(Element element) throws ParseException {
    try {
      Util.getMarshaller().marshal(toJAXBImpl(), element);
    } catch(JAXBException ex) {
      throw new ParseException(ex.getMessage(), -1);
    }
  }

  /**
    * FOR XML STUFF
    */
  private void init(AkentiRequestsGenerated impl) throws IOException, ParseException {
    List temp = impl.getAkentiRequest();
    Iterator iter = temp.listIterator();

    requests = new ArrayList(temp.size());

    do {
      AkentiRequestType req = (AkentiRequestType) iter.next();

      if (req instanceof CheckAccessRequestType) {
        requests.add(new CheckAccessRequest((CheckAccessRequestType) req));
      } else if (req instanceof CheckPolicyRequestType) { 
        requests.add(new CheckPolicyRequest((CheckPolicyRequestType) req));
      } else if (req instanceof ResourceBrowseRequestType) { 
        requests.add(new ResourceBrowseRequest((ResourceBrowseRequestType) req));
      } else if (req instanceof CreateResourceRequestType) { 
        requests.add(new CreateResourceRequest((CreateResourceRequestType) req));
      } else if (req instanceof UploadPolicyCertificateRequestType) { 
        requests.add(new UploadPolicyCertificateRequest((UploadPolicyCertificateRequestType) req));
      } else if (req instanceof GetResourceInfoRequestType) { 
        requests.add(new GetResourceInfoRequest((GetResourceInfoRequestType) req));
      } else {
        throw new RuntimeException();
      }
    } while (iter.hasNext());

    requests = Collections.unmodifiableList(requests);
  }

  /** 
    * FOR XML STUFF
    */
  private AkentiRequestsGenerated toJAXBImpl() throws JAXBException {
    AkentiRequestsGenerated impl = Util.getObjectFactory().createAkentiRequestsGenerated();
    Iterator iter = getAkentiRequests();
    List temp = impl.getAkentiRequest();

    do {
      AkentiRequest ar = (AkentiRequest) iter.next();

      temp.add(ar.toJAXBImpl(false));
    } while (iter.hasNext());

    return impl;
  }
}
