package lbl.akenti.messaging;

import java.util.List;
import java.util.Iterator;
import java.util.ArrayList;
import java.util.Collections;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.text.ParseException;
import java.io.IOException;
import javax.xml.bind.JAXBException;
import lbl.akenti.jaxb.CertificateInfoType;
import lbl.akenti.jaxb.AkentiMsgType;
import lbl.akenti.certs.AkentiCertificate;
import lbl.akenti.certs.PolicyCertificate;
import lbl.akenti.certs.UseCondCertificate;
/**
 * Consists of an <i>AkentiCertificate</i> and an optional <i>AkentiMessage</i>
 * that describes whether the certificate was used or rejected by the <i>AkentiEngine</i>
 *
 * @author Abdelilah Essairi.
 */

final public
class CertificateInfo {
  private AkentiCertificate cert;
  private List mesgsList;
  private boolean used;
  private String location;
  
  /**
   * Constructs this <i>CertificateResponse</i> with a message.
   *
   * @see AkentiCertificate
   * @see AkentiMessage
   */
  public CertificateInfo(AkentiCertificate cert, AkentiMessage[] mesgs, String location, boolean used) {
    if (cert == null || mesgs == null || location == null) {
      throw new IllegalArgumentException();
    }
    
    this.cert = cert;
    this.location = location;
    this.used = used;
    
    mesgsList = new ArrayList(mesgs.length);
    for (int i = 0; i < mesgs.length; ++i) {
      mesgsList.add(mesgs[i]);
    }
      
      mesgsList = Collections.unmodifiableList(mesgsList);
  }
  
  /**
   * Returns the <i>AkentiCertificate</i>.
   *
   * @see AkentiCertificate
   */
  public AkentiCertificate getAkentiCertificate() {
    return cert;
  }
  
  /**
   * Returns the used attribute.
   *
   */
  public boolean isUsed() {
    return used;
  }
  
  /**
   * Returns the location.
   *
   */
  public String getLocation() {
    return location;
  }
  
  /**
   * Returns <i>true</i> if at least one <i>AkentiMessage</i> is present.
   *
   * @see getAkentiMessages
   */
  public boolean hasAkentiMessages() {
    return mesgsList != null;
  }
  
  /**
   * Returns the <i>AkentiMessage</i>s.
   *
   * @see hasAkentiMessage
   */
  public AkentiMessage[] getAkentiMessages() {
    return (AkentiMessage[]) mesgsList.toArray(new AkentiMessage[mesgsList.size()]);
  }
  
  /**
   * Returns a string with basic information; used by GUI Display tool
   *
   */
  public String toString() {
    if (cert.getType().equals("UseCondition")) {
      return "UseCondition for " + ((UseCondCertificate) cert).getResource();
    } else if (cert.getType().equals("Policy")) {
      return "Policy for " + ((PolicyCertificate) cert).getResource();
    } else {
      return "Undefined toString for certificate type";
    }
  }
  
  /**
   * JAXB  STUFF.
   * TODO: Efficiency
   */
  CertificateInfo(CertificateInfoType impl)
  throws IOException, ParseException {
    cert = OpaqueCertificate.fromJAXBImpl(impl.getOpaqueCertificate());
    
    List tempList = impl.getCertificateMessage();
    location = impl.getLocation();
    used = impl.isUsed();
    
    if (tempList != null) {
      Iterator iter = tempList.iterator();
      
      mesgsList = new ArrayList(tempList.size());
      
      while (iter.hasNext()) {
        AkentiMsgType art = (AkentiMsgType) iter.next();
        mesgsList.add(new AkentiMessage(art));
      }
      
      mesgsList = Collections.unmodifiableList(mesgsList);
    }
  }
  
  /**
   * JAXB STUFF.
   * TODO: Efficiency
   */
  CertificateInfoType toJAXBImpl() throws JAXBException {
    CertificateInfoType impl = Util.getObjectFactory().createCertificateInfoType();
    
    impl.setOpaqueCertificate(OpaqueCertificate.toJAXBImpl(cert));
    impl.setUsed(used);
    impl.setLocation(location);
    
    if (hasAkentiMessages()) {
      List tempList = impl.getCertificateMessage();
      Iterator iter = mesgsList.iterator();
      
      while (iter.hasNext()) {
        AkentiMessage art = (AkentiMessage) iter.next();
        tempList.add(art.toJAXBImpl());
      }
            
    }
    
    return impl;
  }
}