package lbl.akenti.policyEngine.impl;

import java.util.List;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.Collections;
import java.util.Set;
import java.util.TreeSet;
import java.security.cert.X509Certificate;
import lbl.akenti.policyEngine.ResourcePolicy;
import lbl.akenti.certs.AttributeCertificate;
import lbl.akenti.certs.CapabilityCertificate;
import lbl.akenti.certs.PolicyCertificate;
import lbl.akenti.certs.Certificate;
import lbl.akenti.certs.elements.AkentiPrincipal;
import lbl.akenti.certs.elements.ConditionalActions;
import lbl.akenti.util.AkentiMessage;

/**
  * Holds information collected during CheckAccess/CheckPolicy requests.<BR>
  * 
  * @author Abdelilah Essiari
  */
final
class AkentiPolicyContext 
  implements lbl.akenti.policyEngine.PolicyContext {

  /**
    * Name of the resource handed to the engine. 
    */
  private String resource;

  /**
    * Name of the resource handed to the engine. 
    */
  private String resAltName;

  /**
    * Subject accessing the resource.
    */
  private AkentiPrincipal subject;

  /**
    * Subject's <i>X509</i> certificat.
    */
  private X509Certificate subjectCert;

  /**
    * Subject's <i>Capability</i> certificate.
    */
  private CapabilityCertificate capCert;

  /**
    * List of <i>Policy</i> and <i>UseCondition</i> certificates collected.
    */
  private List policies = new ArrayList();

  /**
    * List of <i>Attribute</i> certificates collected.
    */
  private List attrCerts = new ArrayList();

  /**
    * List of policies collected.
    */
  private List x509Certs = new ArrayList();

  /**
    * List of certs collected by the external evaluator.
    */
  private List extCerts = new ArrayList();

  /**
    */
  private Set desiredActions  = new TreeSet();

  /**
    */
  private List grantedActions  = new ArrayList();

  /**
    */
  private List condActionsList  = new ArrayList();

  /**
    */
  private AkentiMessage mesg = new AkentiMessage();

  /**
    * Constructs a policy context. 
    */
  AkentiPolicyContext() {
  }

  /**
    * Sets the name of the resource.
    */
  void setResource(String resource) {
    this.resource = resource;
  }

  /**
    * Returns the name of the resource.
    */
  public String getResource()  {
    return resource;
  }

  /**
    * Sets the name of the resource.It could be different
    * the <i>resource</i> name exposed to the outside.
    */
  void setResourceAltName(String resAltName) {
    this.resAltName = resAltName;
  }

  /**
    * Returns the resource alternative name.
    */
  public String getResourceAltName()  {
    return resAltName;
  }

  /**
    * Sets the <i>AkentiPrincipal</i> that is accessing 
    * the resource.
    */
  void setSubject(AkentiPrincipal subject) {
    this.subject = subject;
  }

  /**
    * Returns the <i>AkentiPrincipal</i> that is accessing 
    * the resource. 
    */
  public AkentiPrincipal getSubject()  {
    return subject;
  }

  /**
    * Sets the <i>X509</i> certificate of the entity that is accessing 
    * the resource.
    */
  void setSubjectCertificate(X509Certificate subjectCert) {
    this.subjectCert = subjectCert;
  }

  /**
    * Returns the <i>X509</i> certificate of the entity that is accessing 
    * the resource.
    */
  public X509Certificate getSubjectCertificate() {
    return subjectCert;
  }

  /**
    * Sets the <i>Capability</i> certificate of the entity that is accessing 
    * the resource.
    */
  void setCapabilityCertificate(CapabilityCertificate capCert) {
    this.capCert = capCert;
  }

  /**
    * Returns the <i>Capability</i> certificate of the entity that is accessing 
    * the resource.
    */
  public CapabilityCertificate getCapabilityCertificate()  {
    return capCert;
  }

  /**
    */
  public ResourcePolicy[] toResourcePolicies() {
    return (ResourcePolicy[]) policies.toArray(new ResourcePolicy[0]);
  }

  /**
    * Returns an iterator over the <i>ResourcePolicies</i>
    *
    * @see ResourcePolicy
    */
  Iterator getResourcePolicies()  {
    return policies.iterator();
  }

  /**
    */
  ResourcePolicy getResourcePolicy(int index)  {
    return (ResourcePolicy) policies.get(index);
  }

  /**
    * Adds a <i>ResourcePolicy</i>.
    *
    * @see ResourcePolicy
    */
  void addResourcePolicy(ResourcePolicy policy) {
    policies.add(policy);
  }

  /**
    * Adds an <i>Akenti</i> certificate.
    *
    * @see java.security.certs.X509Certificate
    */
  void addX509Certificate(X509Certificate cert) {
    x509Certs.add(cert);
  }

  /**
    */
  public X509Certificate[] toX509Certificates() {
    return (X509Certificate[]) x509Certs.toArray(new X509Certificate[0]);
  }

  /**
    * Returns an iterator over <i>X509</i> certificates.
    *
    * @see java.security.certs.X509Certificate
    */
  Iterator getX509Certificates() {
    return x509Certs.iterator();
  }
  
  /**
    */
  void addAttributeCertificate(AttributeCertificate attrCert) {
    attrCerts.add(attrCert);
  }

  public AttributeCertificate[] toAttributeCertificates() {
    return (AttributeCertificate[]) attrCerts.toArray(new AttributeCertificate[0]);
  }

  /**
    * Returns an iterator over <i>Attribute</i> certificates</i>.
    */
  Iterator getAttributeCertificates() { 
    return attrCerts.iterator();
  }

  public long getCacheInterval() {
    Iterator iter = getResourcePolicies();

    while (iter.hasNext()) {
      ResourcePolicy po = (ResourcePolicy) iter.next();
      PolicyCertificate cert = po.getPolicyCertificate();
      long cacheTime = cert.getCacheTime();
      
      if (cacheTime >= 0) {
        return cert.getCacheTime();
      }
    }

    return 0;
  }

  public Certificate[] toExternalCertificates() {
    return (Certificate[]) extCerts.toArray(new Certificate[0]);
  }

  void setMessage(int code, String message) {
    mesg = new AkentiMessage(code, message);
  }

  public AkentiMessage getAkentiMessage() {
    return mesg;
  }

  void addExternalCertificate(Certificate cert) {
    extCerts.add(cert);
  }

  /**
    */
  public String[] toDesiredActions() {
    return (String []) desiredActions.toArray(new String[0]);
  }
 
  /**
    */
  boolean containsDesiredActions() {
    return desiredActions.size() != 0;
  }

  /**
    */
  void setDesiredActions(Set desiredActions) {
    if (desiredActions != null) {
      this.desiredActions = desiredActions;
    }
  } 

  /**
    */
  Set getDesiredActions() {
    return desiredActions;
  } 

  void addGrantedActions(Set actions) {
    grantedActions.addAll(actions);
  }

  void addGrantedAction(String action) {
    grantedActions.add(action);
  }

  void addConditionalActions(ConditionalActions condActions) {
    condActionsList.add(condActions);
  }
}
