package lbl.akenti.policyManager.widgets;

/*
 *
 * This widget is used to display text. It consists of at most two
 * components. The first component (optional) is used to display a title.
 * The second component is used to display a given message.
 *
 * @author Abdelilah Essiari
 */
import javax.swing.JPanel;
import javax.swing.JTextArea;
import javax.swing.BorderFactory;
import javax.swing.JLabel;
import java.awt.Font;
import java.awt.BorderLayout;
import java.awt.FontMetrics;
import java.awt.Color;
import java.awt.Toolkit;

public class AKTxtCanvas extends JPanel {
    /**
     */
    public static final Color B_COLOR = new Color(255, 255, 204);
    
    /**
     * Contructs a <code>AKTextCanvas</code> with no title.
     *
     * @param text to be displayed
     */
    public AKTxtCanvas(String text) {
        this(null, text);
    }
    
    /**
     * Contructs a <code>AKTxtCanvas</code>.
     *
     * This <code>>AKTxtCanvas</code> will contain a <code>JLabel</code>
     * if a title is given and a <code>JTextArea</code> where the text
     * will be displayed.
     *
     * @param title to be displayed in a <code>JLabel</code>
     * @param text to displayed in a <code>JTEXTArea</code>
     * @see #createTitleLabel(String text)
     * @see #createTextArea(String text)
     */
    public AKTxtCanvas(String title, String text) {
        super(new BorderLayout(5, 5));
        
        if (title != null) {
            this.add(createTitleLabel(title), "North");
        }
        
      /*
       * we put the area in a panel because setting the border
       * for the area confuses the JTextArea#getPreferredSize
       */
        JPanel panel = new JPanel();
        
        panel.setBackground(B_COLOR);
        panel.setBorder(BorderFactory.createEmptyBorder(3, 3, 3, 3));
        panel.add(createTextArea(text));
        this.add(panel, "Center");
        this.setBorder(BorderFactory.createLineBorder(Color.black, 2));
        this.setBackground(B_COLOR);
    }
    
    /**
     * Creates a <code>JLabel</code> label.
     *
     * @param text to be displayed on the label.
     * @return JLabel.
     */
    private JLabel createTitleLabel(String text) {
        JLabel label = new JLabel(text);
        
        label.setHorizontalAlignment(JLabel.CENTER);
        label.setFont(new Font(label.getFont().getName(), Font.BOLD, 14));
        label.setForeground(Color.black);
        return label;
    }
    
    /**
     *  Create a <code>JTextArea</code> and append the <code>String</code>
     *  argument.
     *
     *  The rows and columns are set according to the size of the argument:
     *     columns will either be set to 20 or 40.
     *     rows will be set to whatever makes the argument fit into the
     *     <code>JTextArea</code>.
     * @param text to be displayed
     * @return JTextArea
     */
    private JTextArea createTextArea(String text) {
        JTextArea area = new JTextArea();
        
        area.setEditable(false);
        area.setWrapStyleWord(true);
        area.setLineWrap(true);
        area.setForeground(Color.black);
        area.setOpaque(false);
        area.append(text);
        
        Font ft = new Font(area.getFont().getName(), Font.PLAIN, 12);
        
        area.setFont(ft);
        
        FontMetrics metrics = Toolkit.getDefaultToolkit().getFontMetrics(ft);
        
        int len = metrics.stringWidth(text);
        int charlen =  metrics.charWidth('m');
        int columns = 40;
        int rows = (int) Math.ceil(((double) len)  / (columns * charlen));
        
        if (rows <= 1) {
            columns = 20;
            rows = (int) Math.ceil(((double) len) / (columns * charlen));
        }
        
        area.setColumns(columns);
        
        for (int i = 0; i < text.length(); i++) {
            if (text.charAt(i)  == '\n') {
                rows++;
            }
        }
        
        area.setRows(rows);
        return area;
    }
}
