package lbl.akenti.policyManager.widgets;

import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.JComponent;
import javax.swing.BorderFactory;
import javax.swing.border.Border;
import javax.swing.JButton;
import javax.swing.Box;
import javax.swing.BoxLayout;
import java.awt.Color;
import java.awt.BorderLayout;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.Container;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.Insets;

/**
 * Superclass of all top level frames that are  associated with a controller.
 *
 * A border layout is used.
 *
 * The north is used for details about the frame.
 * The south is used for the navigation buttons
 * and the help button.
 */
abstract public class LBLFrame extends JFrame {
    public static final int FIRST = 0;
    public static final int MIDDLE= 1;
    public static final int LAST =  2;
    
    private LBLButtonPanel buttonPanel;
    private Controller controller;
    
    /**
     * Constructor
     * Used by Frames in the middle of the list of frames.
     *
     * @param title title of the frame
     * @param header header/instructions to be displayed at top of frame
     */
    protected LBLFrame(String title, String header) {
        this(title, header, MIDDLE);
    }
    
    protected LBLFrame(String title) {
        this(title, null, MIDDLE);
    }
    
    /**
     * Constructor that actually sets up the frame.
     * All frames are the same except the first and last
     * one which have slightly different button configurations
     * at the bottom.
     */
    protected LBLFrame(String title, String header, int type) {
        super(title);
        
        JPanel contentPane = (JPanel) this.getContentPane();
        
        contentPane.setLayout(new BorderLayout(5, 5));
        contentPane.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
        
        if (header != null) {
            contentPane.add(LBLLabelFactory.createHtmlLabel(header), "North");
        }
        
        String [] labels = getButtonLabels(type);
        JButton button = null;
        
        buttonPanel = new LBLButtonPanel();
        
        for (int i = 0; i < labels.length; i++) {
            button = buttonPanel.addButton(labels[i]);
        }
        
        button.setForeground(Color.white);
        button.setBackground(Color.blue);
        buttonPanel.setActionListener(getActionListener());
        contentPane.add(buttonPanel, "South");
    }
    
    /**
     */
    abstract protected void clear();
    
    /**
     */
    protected String [] getButtonLabels(int type) {
        String [] labels;
        
        switch(type) {
            case FIRST:
                labels = new String[] { "Help", "Exit", "Next" };
                break;
            case MIDDLE:
                labels = new String[] { "Help", "Back", "Cancel", "Next" };
                break;
            case LAST:
                labels = new String[] { "Help", "Back", "Cancel", "Sign and Save"};
                break;
            default:
                throw new IllegalArgumentException();
        }
        
        return labels;
    }
    
    /**
     */
    protected ActionListener getActionListener() {
        return new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                String cmd = evt.getActionCommand();
                
                if (cmd.equals("Next")) {
                    getController().showNext();
                } else if (cmd.equals("Back")) {
                    getController().showPrevious();
                } else if (cmd.equals("Cancel")) {
                    getController().cancel();
                } else if (cmd.equals("Sign and Save")) {
                    getController().signAndSave();
                } else if (cmd.equals("Exit")) {
                    getController().exit();
                } else if (cmd.equals("Help")) {
                    showDialog("Info", getController().getHelpMessage());
                } else {
                    throw new RuntimeException();
                }
            }
        };
    }
    
    /**
     */
    protected void setController(Controller controller) {
        this.controller = controller;
    }
    
    /**
     */
    protected Controller getController() {
        return controller;
    }
    
    /**
     */
    protected  void showError(String message) {
        showDialog("Error", message);
    }
    
    /**
     */
    protected  void showInfo(String message) {
        showDialog("Info", message);
    }
    
    /**
     */
    protected void showDialog(String title, String info) {
        LBLDialogFactory.createInfoDialog(
        this,
        title,
        (info == null? "" : info)
        ).setVisible(true);
    }
    
    /**
     */
    protected JButton getButton(String cmd) {
        return buttonPanel.getButton(cmd);
    }
    
    /**
     * Creates a panel with a label followed by a component on each line
     * labels[0]  components[0]
     * labels[1]  components[1] ...
     */
    protected JPanel wrapComponents(JLabel []labels, JComponent []components) {
        return WrapUtil.wrapComponents(labels, components);
    }
    
    /**
     * Wraps two components horizontally next to each other
     */
    protected JPanel wrapComponents(JComponent comp1, JComponent comp2) {
        return WrapUtil.wrapComponents(comp1, comp2);
    }
    
    /**
     * wraps two components vertically on top of each other
     */
    protected JPanel wrapVertical(JComponent comp1, JComponent comp2) {
        return WrapUtil.wrapVertical(comp1, comp2);
    }
    
    /**
     */
    protected void enableBlueButton(String cmd) {
        enableBlueButton(getButton(cmd));
    }
    
    /**
     */
    protected void disableBlueButton(String cmd) {
        disableBlueButton(getButton(cmd));
    }
    
    /**
     */
    protected void enableRedButton(String cmd) {
        enableRedButton(getButton(cmd));
    }
    
    /**
     */
    protected void disableRedButton(String cmd) {
        disableRedButton(getButton(cmd));
    }
    
    protected void enableBlueButton(JButton button) {
        LBLButtonPanel.enableBlueButton(button);
    }
    
    /**
     */
    protected void disableBlueButton(JButton button) {
        LBLButtonPanel.disableBlueButton(button);
    }
    
    /**
     */
    protected  void enableRedButton(JButton button) {
        LBLButtonPanel.enableRedButton(button);
    }
    
    /**
     */
    protected void disableRedButton(JButton button) {
        LBLButtonPanel.disableRedButton(button);
    }
}
