package lbl.akenti.tools;

import java.io.FileInputStream;
import java.io.File;
import java.io.ByteArrayOutputStream;
import java.io.FileOutputStream;
import java.security.PrivateKey;
import java.security.Security;

import util.Arguments;  // from xerces samples

import lbl.akenti.certs.AkentiCertificateFactory;
import lbl.akenti.certs.AkentiCertificate;
import lbl.akenti.certs.AttributeCertificate;
import lbl.akenti.certs.AttributeCertGenerator;
import lbl.akenti.certs.AkentiCertGenerator;
import lbl.akenti.certs.UseCondCertGenerator;
import lbl.akenti.certs.UseCondCertificate;
import lbl.akenti.certs.PolicyCertificate;
import lbl.akenti.certs.PolicyCertGenerator;
import lbl.akenti.certs.elements.CertificateAuthorityInfo;
import lbl.akenti.certs.elements.AkentiPrincipal;
import lbl.akenti.certs.elements.AkentiIssuer;
import lbl.akenti.certs.elements.Condition;
import lbl.akenti.util.P12Reader;
import lbl.akenti.util.Util;
import lbl.akenti.util.UTCTime;

/**
  *  Parses an XML certificate, puts it in the canonical format, signs
  *  it and writes out a new XML and Akenti blob certificate.
  *
  *  @version $id$
  */
public class CertGen {
  public static void main(String[] argv) {

    try {
      Security.addProvider(new com.sun.net.ssl.internal.ssl.Provider());
      java.util.Properties props = System.getProperties();

      props.setProperty("MD5withRSA", "RSA-MD5");

      Arguments argopt = new Arguments();

      argopt.setUsage( 
        new String[] {
          "\nusage: certGen.sh -k keyfile -p password -o outfile [-d days] <xmlcert>\n" +
          "" +
          "options:\n" + 
          "  -k keyfile PKCS12 input filename\n" + 
          "  -p password used to decrypt PKCS12 file\n" +
          "  -o outfile  used to store new certificate\n" +
          "  -d validity period in days\n"
        } 
      );

      if (argv.length == 0) {
        argopt.printUsage();
        System.exit(1);
      }

      argopt.parseArgumentTokens(argv , new char[] {'d', 'k', 'p', 'o'} );

      String keyFileName = null;  
      String password = null;
      String outCertFile = null; 
      String inCertFile = null;
      int period = -1;
      int c;
 
      while ((c =  argopt.getArguments()) != -1) {
        switch (c) {
          case 'k':
            keyFileName = argopt.getStringParameter();
            break;
          case 'p':
            password = argopt.getStringParameter();
            break;
          case 'o':
            outCertFile = argopt.getStringParameter(); 
            break;
          case 'd':
            period = new Integer(argopt.getStringParameter()).intValue();
            break;
          case '-':
          case '?':
            argopt.printUsage();
            System.exit(1);
          default:
            break;
        }
      }

      if (keyFileName == null) {
        System.out.println("Must provide a p12 file");
        argopt.printUsage();
        System.exit(1); 
      } else if (password == null) {
        System.out.println("Must provide a password");
        argopt.printUsage();
        System.exit(1); 
      }

      P12Reader p12 = new P12Reader();

      System.out.println("Loading p12 file from: " + keyFileName);
      p12.readP12File(keyFileName, password);      

      if ((inCertFile = argopt.getlistFiles()) == null) {
        System.out.println("Must provide an xml input file");
        argopt.printUsage();
        System.exit(1); 
      } 

      System.out.println("XML file being parsed: " + new File(inCertFile).getAbsolutePath()); 

      AkentiCertificate inCert = AkentiCertificateFactory.initCertFromFile(
        "XML", inCertFile
      );

      String type = inCert.getType(); 
      AkentiCertificate outCert = null;

      if (type.equals("Attribute")) {
        outCert = generateAttributeCertificate((AttributeCertificate) inCert, p12, period);
      } else if (type.equals("UseCondition")) {
        outCert = generateUseCondCertificate((UseCondCertificate) inCert, p12, period);
      } else if (type.equals("Policy")) {
        outCert = generatePolicyCertificate((PolicyCertificate)inCert, p12, period);
      } else { 
        throw new Exception("Can't handle type " + type);
      }
      
      if (outCertFile == null) {
        System.out.println("Must provide an output file name");
        argopt.printUsage();
        System.exit(1); 
      }
      
     // Don't add hash to file name for policy certificates
      if (!type.equals("Policy")){
	outCertFile += '.' + Util.createHashName(outCert.getHashString()) + ".";
	outCertFile = Util.createFileName(outCertFile);

      }
      System.out.println("Writing cert to " + new File(outCertFile).getAbsolutePath());

      FileOutputStream fos = new FileOutputStream(new File(outCertFile));
      AkentiCertificateFactory.writeCertToStream("XML", outCert, fos);

      fos.close();
    } catch(RuntimeException ex) {
      System.out.println("SHOULD NEVER HAPPEN");
      ex.printStackTrace();
    } catch(Exception ex) {
      ex.printStackTrace();
    }
  } 

  /**
    * Generate the xml file for the AttributeCertificate.
    */
  private static AttributeCertificate
  generateAttributeCertificate(AttributeCertificate inCert, P12Reader p12, int period) 
    throws Exception {
    AttributeCertGenerator gen = new AttributeCertGenerator();

    gen.setVersionNumber(2);
    gen.setUniqueID(computeUniqueID(inCert));

    AkentiPrincipal issuer = new AkentiPrincipal(p12.getSubjectDN(), p12.getIssuerDN());

    gen.setIssuer(new AkentiIssuer(issuer));

    UTCTime[] temp = computeValidityTimes(inCert, period);

    gen.setNotBefore(temp[0]);
    gen.setNotAfter(temp[1]);
    
    gen.setPrincipal(inCert.getPrincipal());
    gen.setAttribute(inCert.getAttribute());
    gen.setValue(inCert.getValue());
 
    Condition cond = inCert.getCondition();

    if (cond != null) {
      gen.setCondition(inCert.getCondition());
    }

    PrivateKey privateKey = p12.getPrivateKey();

    gen.sign("MD5withRSA", privateKey); 
    return (AttributeCertificate) gen.generate();
  }

  /**
    * Generate the xml file for the UseCondCertificate.
    */
  private static UseCondCertificate 
  generateUseCondCertificate(UseCondCertificate inCert, P12Reader p12, int period)
    throws Exception {
    UseCondCertGenerator gen = new UseCondCertGenerator();

    gen.setVersionNumber(2);
    gen.setUniqueID(computeUniqueID(inCert));

    AkentiPrincipal issuer = new AkentiPrincipal(p12.getSubjectDN(), p12.getIssuerDN());

    gen.setIssuer(new AkentiIssuer(issuer));

    UTCTime[] temp = computeValidityTimes(inCert, period);

    gen.setNotBefore(temp[0]);
    gen.setNotAfter(temp[1]);
    gen.setResource(inCert.getResource());
    gen.setCritical(inCert.isCritical());
    gen.setLocalScope(inCert.isScopeLocal());
    gen.setCondition(inCert.getCondition());
    gen.setActions(inCert.getActions());

    PrivateKey privateKey = p12.getPrivateKey();

    gen.sign("MD5withRSA", privateKey);
    return (UseCondCertificate) gen.generate();
  }

  /**
    * Generate the xml file for the PolicyCertificate.
    */
  private static PolicyCertificate 
  generatePolicyCertificate(PolicyCertificate inCert,P12Reader p12, int period)
  throws Exception {

    PolicyCertGenerator gen = new PolicyCertGenerator();

    gen.setVersionNumber(2);
    gen.setUniqueID(computeUniqueID(inCert));

    AkentiPrincipal issuer = new AkentiPrincipal(p12.getSubjectDN(), p12.getIssuerDN());

    gen.setIssuer(new AkentiIssuer(issuer));

    UTCTime[] temp = computeValidityTimes(inCert, period);

    gen.setNotBefore(temp[0]);
    gen.setNotAfter(temp[1]);
    gen.setResource(inCert.getResource());

    CertificateAuthorityInfo[] cais = inCert.getCertificateAuthorityInfos();

    if (cais.length != 0) {
      gen.setCertificateAuthorityInfos(cais);  
    }
   
    gen.setUseCondIssuerGroups(inCert.getUseCondIssuerGroups());
    
    lbl.akenti.certs.elements.Directory[] attrDirs = inCert.getAttributeDirectories();

    if (attrDirs.length != 0) {  
      gen.setAttributeDirectories(attrDirs);
    } 

    gen.setCacheTime(inCert.getCacheTime());

    PrivateKey privateKey = p12.getPrivateKey();

    gen.sign("MD5withRSA", privateKey);
    return (PolicyCertificate) gen.generate();
  }
  
  /**
    * Uses the period supplied by the user otherwise it Uses the
    * <i>ValidityTimes</i> from the supplied certificate.
    */
  private static UTCTime[] 
  computeValidityTimes(AkentiCertificate cert, int period) 
    throws Exception {
    UTCTime now = new UTCTime();

    long l1 = cert.getNotBefore().getTime();
    long l2 = cert.getNotAfter().getTime();

    if (period < 0) {
      UTCTime notBefore = cert.getNotBefore();
      
      if (notBefore.compareTo(now) < 0) {
        notBefore = now;
      }

      UTCTime notAfter = cert.getNotAfter();

      if (notBefore.compareTo(notAfter) >= 0) {
        throw new Exception("Bad expiration time");
      }

      return new UTCTime[] { notBefore, notAfter};
    }

    return new UTCTime[] { now, new UTCTime(now, 1000L * 60 * 60 * 24 * period) };
  }

  /**
    * Computes the unique id.
    */
  private static String computeUniqueID(AkentiCertificate cert) {
    return Util.createUniqueID(Util.createHashName(cert.getHashString()));
  }
}
