package lbl.akenti.util;

import java.security.PrivateKey;
import java.security.cert.Certificate;
import java.security.cert.X509Certificate;
import java.security.GeneralSecurityException;
import java.security.KeyStore;
import java.security.Principal;

import java.io.FileInputStream;
import java.io.IOException;
import java.util.Enumeration;

/**
  * A class to simplify reading private keys and certificates from
  * a netscape .p12 file.  
  *
  * @author Keith Jackson
  * @author Abdelilah Essiari
  */
public class P12Reader {

    private Certificate[] certs;
    private PrivateKey key;

    /**
      */
    public P12Reader() {
    }

    /**
      * Loads the x509 certs and the private key from a netscape .p12 file.
      *
      * @see readP12File(String, char[])
      */
    public void readP12File(String fileName, String password) 
      throws IOException, GeneralSecurityException {
      readP12File(fileName, password.toCharArray());
    }

    /**
      * Loads the x509 certs and the private key from a netscape .p12 file.
      *
      * @param fileName The path to the .p12 file.
      * @param password The password to decrypt the p12 file.
      *
      * @exception FileNotFoundException if unable to locate the file
      * @exception IOException if io error
      * @exception KeyStoreException if unable to find a pkcs12 KeyStore 
      * @exception NoSuchAlgorithException if unable to find  
      *            the integrity algorithm of the keystore.
      * @exception CertificateException if unable to load the x509 certificates
      */
    public void readP12File(String fileName, char[] password) 
      throws IOException, GeneralSecurityException {
      certs = null;
      key = null;

      KeyStore keyStore = KeyStore.getInstance("pkcs12");
      keyStore.load(new FileInputStream(fileName), password);

      Enumeration e = keyStore.aliases();

      String alias = (String) e.nextElement();

      certs = (Certificate[]) keyStore.getCertificateChain(alias);
      key =   (PrivateKey) keyStore.getKey(alias, password);
   }

   /**
     */
   public Principal getSubjectDN() {
     X509Certificate cert = (X509Certificate) getX509Certificate();

     return cert.getSubjectDN();
   }

   /**
     */
   public Principal getIssuerDN() {
     X509Certificate cert = (X509Certificate) getX509Certificate();

     return cert.getIssuerDN();
   }

   /**
     * A method to return the users X509 certificates.
     */
   public Certificate getX509Certificate() {
     return certs[0];
   }

   /**
     * A method to return the users X509 certificates.
     */
   public Certificate[] getX509CertificateChain() {
     return certs;
   }

   /**
     * A method to return the users PrivateKey from the p12 file.
     */
    public PrivateKey getPrivateKey() {
      return key;
    }
}  
