package lbl.akenti.certs;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.net.URL;
import java.security.cert.CertificateException;
import java.text.ParseException;
import java.util.Hashtable;

import lbl.akenti.certs.elements.AkentiInputStream;
import lbl.akenti.certs.elements.AkentiOutputStream;
import lbl.akenti.util.Base64;
import lbl.akenti.util.ParsingException;

import lbl.akenti.util.XMLUtil;
import org.apache.xerces.dom.DocumentImpl;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;


/**
  * This class generates the AkentiCertificate objects from input
  * files or streams containing XML or PEM-tagged, base64-encoded Akenti
  * canonical format data; it also contains static methods to write out
  * an Akenti Certificate in either format
  */ 
public class AkentiCertificateFactory {
  private static AkentiCertificateFactory fac; 

  private Hashtable prototypes;

  private AkentiCertificateFactory() {
    prototypes = new Hashtable();
   
    addPrototype(new UseCondCertificate());
    addPrototype(new AttributeCertificate());
    addPrototype(new PolicyCertificate());
    addPrototype(new CapabilityCertificate());
  }

  public void addPrototype(AkentiCertificate cert) {
    prototypes.put(cert.getType().toLowerCase(), cert);
  }

  public AkentiCertificate getPrototype(String type) {
    AkentiCertificate cert = (AkentiCertificate) prototypes.get(type.toLowerCase());
    
    return cert != null? (AkentiCertificate) cert.newInstance() : null;
  }

  /**
    */
  public static AkentiCertificateFactory getInstance() {
    if (fac == null) {
      synchronized(AkentiCertificateFactory.class) {
        if (fac == null) {
          fac = new AkentiCertificateFactory();
        }
      }
    }

    return fac;
  }

  /**
    * DEPRECATED USE getInstance
    */
  public static synchronized AkentiCertificateFactory 
  getAkentiCertificateFactory() {
    return getInstance();
  }

  public String toString() {
    return prototypes.toString();
  }

  public String paramString() {
    return toString();
  }

  private static final String B_HEADER = "-----BEGIN AKENTI ";
  private static final String E_HEADER = "-----END AKENTI ";
  private static final String C_HEADER = " CERTIFICATE-----";

  /**
    *  Parses PEM headers of the form
    *   -----BEGIN/END AKENTI <CertType> CERTIFICATE-----
    *  and returns the the CertType name as a string.
    *
    * @param begin - if true we are looking for a BEGIN tag, otherwise and END tag
    * @param header - A Akenti certficate PEM tag
    *
    * @return The name of the Akenti certificate type
    */
  private static String 
  parseCertHeader(boolean begin, String header) {
    String temp = begin ? B_HEADER : E_HEADER;

    int i = header.indexOf(temp);

    if (i != 0) {
      return null;
    }

    i += temp.length();

    int j = header.indexOf(C_HEADER, i);

    if (j + C_HEADER.length() != header.length()) {
      return null;
    }

    return header.substring(i, j).toLowerCase();
  }

  /**
    */
  public static AkentiCertificate
  initPEMCertFromStream(InputStream is) 
  throws ParseException, IOException {
    BufferedReader reader = new BufferedReader(new InputStreamReader(is));
    String line, beginType = null;

    do {
      line = reader.readLine();
    } while (line != null && (beginType = parseCertHeader(true, line)) == null);

    if (line == null) { 
      throw new ParsingException("proper begin header not encountered");
    }

    line = reader.readLine();

    StringBuffer base64 = new StringBuffer();
    String endType = null;

    while (line != null && (endType = parseCertHeader(false, line)) == null) {
      base64.append(line);
      base64.append('\n');
      line = reader.readLine();
    }

    if (line == null) {
      throw new ParsingException("proper end header not encountered");
    } else if (!beginType.equals(endType)) {
      throw new ParsingException("begin/end headers dont match");
    }

    AkentiCertificateFactory fac = getAkentiCertificateFactory();
    AkentiCertificate cert = fac.getPrototype(beginType);
      
    if (cert == null) {
      throw new ParsingException("Unsupported certificate type " + beginType);
    }

    byte[] certbytes = Base64.decode(base64.toString());
    ByteArrayInputStream bins = new ByteArrayInputStream(certbytes);
    AkentiInputStream abins = new AkentiInputStream(bins);

    cert.readObject(abins);
    return cert;
  }

  public static AkentiCertificate
  initXMLCertFromStream(InputStream is) 
  throws ParseException, IOException {
    Element root = XMLUtil.parseDocument(is).getDocumentElement();
    
    if (root == null) {
      throw new ParsingException("No root element");
    }

    XMLUtil.filter(root);

    NodeList list = root.getElementsByTagName("Header");

    if (list.getLength() != 1) {
      throw new ParsingException("No header");
    }

    Element header = (Element) list.item(0);
    String type = header.getAttribute("Type");
    AkentiCertificate cert = getAkentiCertificateFactory().getPrototype(type);
     
    if (cert == null) {
      throw new ParsingException("Unsupported certificate type " + type);
    }

    cert.fromXML(root);
    return cert;
  }

  /**
    * Reads a <i>AkentiCertficate</i> in either XML or PEM-tagged, base64-encoded
    * signed canonical format from a stream.
    *
    * @param encType "XML" or PEM"
    * @param is stream that contains the certificate
    * @exception IOException
    */ 
  public static AkentiCertificate
  initCertFromStream(String encType, InputStream is) 
  throws ParseException, IOException {
    if (encType.equalsIgnoreCase("PEM")) {
      return initPEMCertFromStream(is);
    } else if (encType.equalsIgnoreCase("XML")) {
      return initXMLCertFromStream(is);
    } else {
      throw new ParsingException("Unsupported encoding type:" + encType);
    }
  }

  /**
    * Reads a <i>AkentiCertficate</i> in either XML or PEM-tagged, base64-encoded
    * signed canonical format from a file. If encType is null then it tries both
    * types.
    *
    * @param encType "XML" or PEM"
    * @param fileName  name of the file that contains the certificate
    *
    * @exception IOException
    */ 
  public static AkentiCertificate 
  initCertFromFile(String encType, String fileName)
  throws ParseException, IOException {
    FileInputStream fin = new FileInputStream(fileName);
    AkentiCertificate cert = null;

    if (encType == null) {   // TRY XML THEN PEM
      try {
        cert = initCertFromStream("XML", fin);
      } catch(IOException ex) {
        fin = new FileInputStream(fileName);
        cert = initCertFromStream("PEM", fin);
      } catch(ParseException ex) {
        fin = new FileInputStream(fileName);
        cert = initCertFromStream("PEM", fin);
      }
    } else {
      cert = initCertFromStream(encType, fin);
    }

    fin.close();
    return cert;
  }

  /**
    * Reads a <i>AkentiCertficate</i> in either XML or PEM-tagged, base64-encoded
    * signed canonical format from a url. Tries both types if no encoding type
    * is specified.
    *
    * @param encType format of the input string (XML or PEM)
    * @param url name of the url containing the certificate data
    * @exception IOException
    */ 
  public static AkentiCertificate 
  initCertFromURL(String encType, URL url)
  throws ParseException, IOException {
    InputStream is = url.openConnection().getInputStream();
    AkentiCertificate cert = null;

    if (encType == null) {
      try {
        cert = initCertFromStream("XML", is);
      } catch(IOException ex) {
        is = url.openConnection().getInputStream();
        cert = initCertFromStream("PEM", is);
      } catch(ParseException ex) {
        is = url.openConnection().getInputStream();
        cert = initCertFromStream("PEM", is);
      }
    } else {
      cert = initCertFromStream(encType, is);
    }

    is.close();
    return cert;
  }

  public static void 
  writePEMCertToStream(AkentiCertificate cert, OutputStream os) 
    throws IOException {
    ByteArrayOutputStream baos = new ByteArrayOutputStream();

      cert.writeObject(new AkentiOutputStream(baos));

    Writer writer = new BufferedWriter(new OutputStreamWriter(os));

    writer.write(B_HEADER);
    writer.write(cert.getType().toUpperCase());
    writer.write(C_HEADER);
    writer.write('\n');
    writer.write(Base64.encode(baos.toByteArray()));
    writer.write('\n');
    writer.write(E_HEADER);
    writer.write(cert.getType().toUpperCase());
    writer.write(C_HEADER);
    writer.write('\n');
    writer.flush();
  }
   
  public static void 
  writeXMLCertToStream(AkentiCertificate cert, OutputStream os) 
    throws IOException {
    Document doc = new DocumentImpl();
    Element akcert = doc.createElement("ak:AkentiCertificate");
    akcert.setAttribute("xmlns:ak", "http://dsd.lbl.gov/Akenti/2004/05/AkentiCertificate");
    
    cert.toXML(akcert);
    doc.appendChild(akcert);
    XMLUtil.writeDocument(doc, os);
  }

  /**
    * Writes an Akenti certificate to an output stream in either XML or 
    * PEM-tagged, base64-encoded signed canonical format.
    *
    * @param enc format desired, either "XML" or PEM"
    * @param cert Akenti Certificate
    * @param os  output stream
    *
    */ 
  public static void 
  writeCertToStream(String encType, AkentiCertificate cert, OutputStream os)
    throws IOException, CertificateException {
    if (encType.equalsIgnoreCase("PEM")) {
      writePEMCertToStream(cert, os);
    } else if (encType.equalsIgnoreCase("XML")) { 
      writeXMLCertToStream(cert, os);
    } else {
      throw new CertificateException("Unsupported encoding format:" + encType);
    }
  }

  /**
    * Writes an Akenti certificate to an file in either XML or 
    * PEM-tagged, base64-encoded signed canonical format.
    *
    * @param enc the format desired, either "XML" or PEM"
    * @param cert the AkentiCertificate object
    * @param fileName the output file name
    */ 
  public static void 
  writeCertToFile(String encType, AkentiCertificate cert, String fileName) 
    throws IOException, CertificateException {
    FileOutputStream fout = new FileOutputStream(fileName);

    writeCertToStream(encType, cert, fout);
    fout.close();
  }
}
